@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2014, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -50,8 +49,8 @@
 // 2) It is parsed and used as default values for all Config.get calls.
 std::string Config::_defaults =
   "# Taskwarrior program configuration file.\n"
-  "# For more documentation, see http://taskwarrior.org or try 'man task', 'man task-faq',\n"
-  "# 'man task-tutorial', 'man task-color', 'man task-sync' or 'man taskrc'\n"
+  "# For more documentation, see http://taskwarrior.org or try 'man task', 'man task-color',\n"
+  "# 'man task-sync' or 'man taskrc'\n"
   "\n"
   "# Here is an example of entries that use the default, override and blank values\n"
   "#   variable=foo   -- By specifying a value, this overrides the default\n"
@@ -65,6 +64,7 @@ std::string Config::_defaults =
   "locking=on                                     # Use file-level locking\n"
   "gc=on                                          # Garbage-collect data files - DO NOT CHANGE unless you are sure\n"
   "exit.on.missing.db=no                          # Whether to exit if ~/.task is not found\n"
+  "hooks=on                                       # Master control switch for hooks\n"
   "\n"
   "# Terminal\n"
   "detection=on                                   # Detects terminal width\n"
@@ -80,15 +80,22 @@ std::string Config::_defaults =
   "#verbose=no                                    # Provide regular feedback\n"
   "#verbose=nothing                               # Provide no feedback\n"
   "#                                              # Comma-separated list.  May contain any subset of:\n"
-  "#verbose=blank,header,footnote,label,new-id,affected,edit,special,project,sync\n"
+  "#verbose=blank,header,footnote,label,new-id,new-uuid,affected,edit,special,project,sync,filter\n"
   "confirmation=yes                               # Confirmation on delete, big changes\n"
+  "recurrence=yes                                 # Enable recurrence\n"
+  "recurrence.confirmation=prompt                 # Confirmation for propagating changes among recurring tasks (yes/no/prompt)\n"
+  "allow.empty.filter=yes                         # An empty filter gets a warning and requires confirmation\n"
   "indent.annotation=2                            # Indent spaces for annotations\n"
   "indent.report=0                                # Indent spaces for whole report\n"
   "row.padding=0                                  # Left and right padding for each row of report\n"
   "column.padding=1                               # Spaces between each column in a report\n"
   "bulk=3                                         # 3 or more tasks considered a bulk change and is confirmed\n"
   "nag=You have more urgent tasks.                # Nag message to keep you honest\n"                      // TODO
+#ifdef CYGWIN
+  "search.case.sensitive=no                       # Case-insensitive regex searches are broken under Cygwin\n"
+#else
   "search.case.sensitive=yes                      # Setting to no allows case insensitive searches\n"
+#endif
   "active.indicator=*                             # What to show as an active task indicator\n"
   "tag.indicator=+                                # What to show as a tag indicator\n"
   "dependency.indicator=D                         # What to show as a dependency indicator\n"
@@ -96,10 +103,9 @@ std::string Config::_defaults =
   "recurrence.limit=1                             # Number of future recurring pending tasks\n"
   "undo.style=side                                # Undo style - can be 'side', or 'diff'\n"
   "burndown.bias=0.666                            # Weighted mean bias toward recent data\n"
-  "regex=no                                       # Assume all search/filter strings are regexes\n"
+  "regex=yes                                      # Assume all search/filter strings are regexes\n"
   "xterm.title=no                                 # Sets xterm title for some commands\n"
-  "expressions=on                                 # Support for algebraic expressions\n"
-  "patterns=on                                    # Support for regex patterns\n"
+  "expressions=infix                              # Prefer infix over postfix expressions\n"
   "dom=on                                         # Support DOM access\n"
   "json.array=off                                 # Enclose JSON output in [ ]\n"
   "abbreviation.minimum=2                         # Shortest allowed abbreviation\n"
@@ -111,6 +117,7 @@ std::string Config::_defaults =
   "dateformat.info=Y-M-D H:N:S                    # Preferred display date format for information\n"
   "dateformat.report=                             # Preferred display date format for reports\n"
   "dateformat.annotation=                         # Preferred display date format for annotations\n"
+  "date.iso=yes                                   # Enable ISO date support\n"
   "weekstart="
              STRING_DATE_SUNDAY_LONG
                   "                               # Sunday or Monday only\n"
@@ -140,7 +147,6 @@ std::string Config::_defaults =
   "urgency.next.coefficient=15.0                  # Urgency coefficient for 'next' special tag\n"
   "urgency.due.coefficient=12.0                   # Urgency coefficient for due dates\n"
   "urgency.blocking.coefficient=8.0               # Urgency coefficient for blocking tasks\n"
-  "urgency.priority.coefficient=6.0               # Urgency coefficient for priorities\n"
   "urgency.active.coefficient=4.0                 # Urgency coefficient for active tasks\n"
   "urgency.scheduled.coefficient=5.0              # Urgency coefficient for scheduled tasks\n"
   "urgency.age.coefficient=2.0                    # Urgency coefficient for age\n"
@@ -148,6 +154,7 @@ std::string Config::_defaults =
   "urgency.tags.coefficient=1.0                   # Urgency coefficient for tags\n"
   "urgency.project.coefficient=1.0                # Urgency coefficient for projects\n"
   "urgency.blocked.coefficient=-5.0               # Urgency coefficient for blocked tasks\n"
+  "urgency.inherit.coefficient=0.0                # Urgency coefficient for blocked tasks inheriting from blocking tasks\n"
   "urgency.waiting.coefficient=-3.0               # Urgency coefficient for waiting status\n"
   "urgency.age.max=365                            # Maximum age in days\n"
   "\n"
@@ -157,263 +164,307 @@ std::string Config::_defaults =
   "\n"
   "# Color controls.\n"
   "color=on                                       # Enable color\n"
-#ifdef LINUX
-  "color.header=color3                            # Color of header messages\n"
-  "color.footnote=color3                          # Color of footnote messages\n"
-  "color.error=color3                             # Color of error messages\n"
-  "color.debug=color3                             # Color of diagnostic output\n"
-  "color.alternate=on color233                    # Alternate color for line coloring\n"
+#if defined(LINUX) || defined(DARWIN)
   "\n"
-  "color.summary.bar=black on rgb141              # Color of summary report progress bar\n"
-  "color.summary.background=white on color0       # Color of summary report background\n"
+  "rule.precedence.color=deleted,completed,active,keyword.,tag.,uda.,project.,overdue,scheduled,due.today,due,blocked,blocking,recurring,tagged,pri.\n"
   "\n"
-  "color.history.add=color0 on rgb500             # Color of added tasks in ghistory report\n"
-  "color.history.done=color0 on rgb050            # Color of completed tasks in ghistory report\n"
-  "color.history.delete=color0 on rgb550          # Color of deleted tasks in ghistory report\n"
+  "# General decoration\n"
+  "color.label=\n"
+  "color.label.sort=\n"
+  "color.alternate=on gray2\n"
+  "color.header=color3\n"
+  "color.footnote=color3\n"
+  "color.warning=bold red\n"
+  "color.error=white on red\n"
+  "color.debug=color4\n"
   "\n"
-  "color.burndown.pending=color0 on rgb500        # Color of pending tasks in burndown report\n"
-  "color.burndown.done=color0 on rgb050           # Color of completed tasks in burndown report\n"
-  "color.burndown.started=color0 on rgb550        # Color of started tasks in burndown report\n"
+  "# Task state\n"
+  "color.completed=\n"
+  "color.deleted=\n"
+  "color.active=rgb555 on rgb410\n"
+  "color.recurring=rgb013\n"
+  "color.scheduled=on rgb001\n"
+  "color.until=\n"
+  "color.blocked=white on color8\n"
+  "color.blocking=black on color15\n"
   "\n"
-  "color.sync.added=rgb005                        # Color of added tasks in sync output\n"
-  "color.sync.changed=rgb550                      # Color of changed tasks in sync output\n"
-  "color.sync.rejected=rgb500                     # Color of rejected tasks in sync output\n"
+  "# Project\n"
+  "color.project.none=\n"
   "\n"
-  "color.undo.before=color1                       # Color of values before a change\n"
-  "color.undo.after=color2                        # Color of values after a change\n"
+  "# Priority UDA\n"
+  "color.uda.priority.H=color255\n"
+  "color.uda.priority.L=color245\n"
+  "color.uda.priority.M=color250\n"
   "\n"
-  "color.calendar.today=color15 on rgb013         # Color of today in calendar\n"
-  "color.calendar.due=color0 on color1            # Color of days  with due tasks in calendar\n"
-  "color.calendar.due.today=color15 on color1     # Color of today with due tasks in calendar\n"
-  "color.calendar.overdue=color0 on color9        # Color of days  with overdue tasks in calendar\n"
-  "color.calendar.weekend=color235                # Color of weekend days in calendar\n"
-  "color.calendar.holiday=color0 on color11       # Color of public holidays in calendar\n"
-  "color.calendar.weeknumber=rgb013               # Color of the weeknumbers in calendar\n"
+  "# Tags\n"
+  "color.tag.next=rgb440\n"
+  "color.tag.none=\n"
+  "color.tagged=rgb031\n"
+  "\n"
+  "# Due\n"
+  "color.due.today=rgb400\n"
+  "color.due=color1\n"
+  "color.overdue=color9\n"
+  "\n"
+  "# Report: burndown\n"
+  "color.burndown.done=on rgb010\n"
+  "color.burndown.pending=on color9\n"
+  "color.burndown.started=on color11\n"
+  "\n"
+  "# Report: history\n"
+  "color.history.add=color0 on rgb500\n"
+  "color.history.delete=color0 on rgb550\n"
+  "color.history.done=color0 on rgb050\n"
+  "\n"
+  "# Report: summary\n"
+  "color.summary.background=white on color0\n"
+  "color.summary.bar=black on rgb141\n"
+  "\n"
+  "# Command: calendar\n"
+  "color.calendar.due.today=color15 on color1\n"
+  "color.calendar.due=color0 on color1\n"
+  "color.calendar.holiday=color0 on color11\n"
+  "color.calendar.overdue=color0 on color9\n"
+  "color.calendar.today=color15 on rgb013\n"
+  "color.calendar.weekend=on color235\n"
+  "color.calendar.weeknumber=rgb013\n"
+  "\n"
+  "# Command: sync\n"
+  "color.sync.added=rgb010\n"
+  "color.sync.changed=color11\n"
+  "color.sync.rejected=color9\n"
+  "\n"
+  "# Command: undo\n"
+  "color.undo.after=color2\n"
+  "color.undo.before=color1\n"
   "\n"
-  "# Here are the color rules.\n"
-  "color.recurring=rgb013                         # Color of recur.any: tasks\n"
-  "color.overdue=color9                           # Color of overdue tasks\n"
-  "color.due.today=rgb400                         # Color of tasks due today\n"
-  "color.due=color1                               # Color of due tasks\n"
-  "#color.keyword.car=on blue                     # Color of description.contains:car tasks\n"
-  "#color.project.garden=on green                 # Color of project:garden tasks\n"
-  "#color.project.none=                           # Color of tasks with no project\n"
-  "#color.tag.bug=yellow                          # Color of +bug tasks\n"
-  "#color.tag.none=                               # Color of tag-less tasks\n"
-  "color.active=rgb555 on rgb410                  # Color of active tasks\n"
-  "color.scheduled=rgb444 on rgb410               # Color of scheduled tasks\n"
-  "color.pri.none=                                # Color of priority:  tasks\n"
-  "color.pri.H=rgb255                             # Color of priority:H tasks\n"
-  "color.pri.M=rgb250                             # Color of priority:M tasks\n"
-  "color.pri.L=rgb245                             # Color of priority:L tasks\n"
-  "color.tagged=rgb031                            # Color of tagged tasks\n"
-  "color.blocked=white on color8                  # Color of blocked tasks\n"
-  "color.blocking=white on color6                 # Color of blocking tasks\n"
-  "#color.completed=on blue                       # Color of completed tasks\n"
-  "#color.deleted=on blue                         # Color of deleted tasks\n"
-  "#color.uda.estimate=on green                   # Color of UDA\n"
 #else
-  "color.header=yellow                            # Color of header messages\n"
-  "color.footnote=yellow                          # Color of footnote messages\n"
-  "color.error=yellow                             # Color of error messages\n"
-  "color.debug=yellow                             # Color of diagnostic output\n"
-  "color.alternate=                               # Alternate color for line coloring\n"
   "\n"
-  "color.summary.bar=black on green               # Color of summary report progress bar\n"
-  "color.summary.background=white on black        # Color of summary report background\n"
+  "rule.precedence.color=deleted,completed,active,keyword.,tag.,uda.,project.,overdue,scheduled,due.today,due,blocked,blocking,recurring,tagged,pri.\n"
   "\n"
-  "color.history.add=black on red                 # Color of added tasks in ghistory report\n"
-  "color.history.done=black on green              # Color of completed tasks in ghistory report\n"
-  "color.history.delete=black on yellow           # Color of deleted tasks in ghistory report\n"
+  "# General decoration\n"
+  "color.label=\n"
+  "color.label.sort=\n"
+  "color.alternate=\n"
+  "color.header=yellow\n"
+  "color.footnote=yellow\n"
+  "color.warning=bold red\n"
+  "color.error=white on red\n"
+  "color.debug=blue\n"
   "\n"
-  "color.burndown.pending=black on red            # Color of pending tasks in burndown report\n"
-  "color.burndown.done=black on green             # Color of completed tasks in burndown report\n"
-  "color.burndown.started=black on yellow         # Color of started tasks in burndown report\n"
+  "# Task state\n"
+  "color.completed=\n"
+  "color.deleted=\n"
+  "color.active=black on bright green\n"
+  "color.recurring=magenta\n"
+  "color.scheduled=white on green\n"
+  "color.until=\n"
+  "color.blocked=black on white\n"
+  "color.blocking=black on bright white\n"
   "\n"
-  "color.sync.added=green                         # Color of added tasks in sync output\n"
-  "color.sync.changed=yellow                      # Color of changed tasks in sync output\n"
-  "color.sync.rejected=red                        # Color of rejected tasks in sync output\n"
+  "# Project\n"
+  "color.project.none=\n"
   "\n"
-  "color.undo.before=red                          # Color of values before a change\n"
-  "color.undo.after=green                         # Color of values after a change\n"
+  "# Priority UDA\n"
+  "color.uda.priority.H=bold white\n"
+  "color.uda.priority.M=white\n"
+  "color.uda.priority.L=\n"
   "\n"
-  "color.calendar.today=bold white on bright blue # Color of today in calendar\n"
-  "color.calendar.due=white on red                # Color of days  with due tasks in calendar\n"
-  "color.calendar.due.today=bold white on red     # Color of today with due tasks in calendar\n"
-  "color.calendar.overdue=black on bright red     # Color of days  with overdue tasks in calendar\n"
-  "color.calendar.weekend=white on bright black   # Color of weekend days in calendar\n"
-  "color.calendar.holiday=black on bright yellow  # Color of public holidays in calendar\n"
-  "color.calendar.weeknumber=bold blue            # Color of the weeknumbers in calendar\n"
+  "# Tags\n"
+  "color.tag.next=bold yellow\n"
+  "color.tag.none=\n"
+  "color.tagged=green\n"
+  "\n"
+  "# Due\n"
+  "color.due=red\n"
+  "color.due.today=red\n"
+  "color.overdue=bold red\n"
+  "\n"
+  "# UDA\n"
+  "#color.uda.X=\n"
+  "\n"
+  "# Report: burndown\n"
+  "color.burndown.done=on green\n"
+  "color.burndown.pending=on red\n"
+  "color.burndown.started=on yellow\n"
+  "\n"
+  "# Report: history\n"
+  "color.history.add=black on red\n"
+  "color.history.delete=black on yellow\n"
+  "color.history.done=black on green\n"
+  "\n"
+  "# Report: summary\n"
+  "color.summary.background=white on black\n"
+  "color.summary.bar=black on green\n"
+  "\n"
+  "# Command: calendar\n"
+  "color.calendar.due=white on red\n"
+  "color.calendar.due.today=bold white on red\n"
+  "color.calendar.holiday=black on bright yellow\n"
+  "color.calendar.overdue=black on bright red\n"
+  "color.calendar.today=bold white on bright blue\n"
+  "color.calendar.weekend=white on bright black\n"
+  "color.calendar.weeknumber=bold blue\n"
+  "\n"
+  "# Command: sync\n"
+  "color.sync.added=green\n"
+  "color.sync.changed=yellow\n"
+  "color.sync.rejected=red\n"
+  "\n"
+  "# Command: undo\n"
+  "color.undo.after=green\n"
+  "color.undo.before=red\n"
   "\n"
-  "# Here are the color rules.\n"
-  "color.recurring=magenta                        # Color of recur.any: tasks\n"
-  "color.overdue=bold red                         # Color of overdue tasks\n"
-  "color.due.today=red                            # Color of tasks due today\n"
-  "color.due=red                                  # Color of due tasks\n"
-  "#color.keyword.car=on blue                     # Color of description.contains:car tasks\n"
-  "#color.project.garden=on green                 # Color of project:garden tasks\n"
-  "#color.project.none=                           # Color of tasks with no project\n"
-  "#color.tag.bug=yellow                          # Color of +bug tasks\n"
-  "#color.tag.none=                               # Color of tag-less tasks\n"
-  "color.active=black on bright green             # Color of active tasks\n"
-  "color.scheduled=black on green                 # Color of scheduled tasks\n"
-  "color.pri.none=                                # Color of priority:  tasks\n"
-  "color.pri.H=bold white                         # Color of priority:H tasks\n"
-  "color.pri.M=white                              # Color of priority:M tasks\n"
-  "color.pri.L=                                   # Color of priority:L tasks\n"
-  "color.tagged=green                             # Color of tagged tasks\n"
-  "color.blocked=black on white                   # Color of blocked tasks\n"
-  "color.blocking=black on bright white           # Color of blocking tasks\n"
-  "#color.completed=on blue                       # Color of completed tasks\n"
-  "#color.deleted=on blue                         # Color of deleted tasks\n"
-  "#color.uda.estimate=on green                   # Color of UDA\n"
 #endif
+  "# UDA priority\n"
+  "uda.priority.type=string                       # UDA priority is a string type\n"
+  "uda.priority.label=Priority                    # UDA priority has a display label'\n"
+  "uda.priority.values=H,M,L,                     # UDA priority values are 'H', 'M', 'L' or ''\n"
+  "                                               # UDA priority sorting is 'H' > 'M' > 'L' > '' (highest to lowest)\n"
+  "#uda.priority.default=M                        # UDA priority default value of 'M'\n"
+  "urgency.uda.priority.H.coefficient=6.0         # UDA priority coefficient for value 'H'\n"
+  "urgency.uda.priority.M.coefficient=3.9         # UDA priority coefficient for value 'M'\n"
+  "urgency.uda.priority.L.coefficient=1.8         # UDA priority coefficient for value 'L'\n"
   "\n"
   "# Here is the rule precedence order, highest to lowest.\n"
   "# Note that these are just the color rule names, without the leading 'color.'\n"
   "#      and any trailing '.value'.\n"
-  "rule.precedence.color=due.today,active,blocking,blocked,overdue,due,scheduled,keyword.,project.,tag.,uda.,recurring,pri.,tagged,completed,deleted\n"
-  "\n"
-  "# Shadow file support\n"
-  "#shadow.file=/tmp/shadow.txt                   # Location of shadow file\n"
-  "#shadow.command=list                           # Task command for shadow file\n"
-  "#shadow.notify=on                              # Footnote when updated\n"
+  "rule.precedence.color=deleted,completed,active,keyword.,tag.,uda.,project.,overdue,scheduled,due.today,due,blocked,blocking,recurring,tagged,pri.\n"
   "\n"
   "#default.project=foo                           # Default project for 'add' command\n"
-  "#default.priority=M                            # Default priority for 'add' command\n"
   "#default.due=eom                               # Default due date for 'add' command\n"
   "default.command=next                           # When no arguments are specified\n"
   "\n"
   "_forcecolor=no                                 # Forces color to be on, even for non TTY output\n"
-  "complete.all.projects=no                       # Include old project names in '_projects' command\n"
   "complete.all.tags=no                           # Include old tag names in '_ags' command\n"
   "list.all.projects=no                           # Include old project names in 'projects' command\n"
+  "summary.all.projects=no                        # Include old project names in 'summary' command\n"
   "list.all.tags=no                               # Include old tag names in 'tags' command\n"
   "print.empty.columns=no                         # Print columns which have no data for any task\n"
   "debug=no                                       # Display diagnostics\n"
-  "debug.tls=0                                    # GnuTLS log level\n"
-  "extensions=off                                 # Extension system master switch\n"
   "fontunderline=yes                              # Uses underlines rather than -------\n"
   "shell.prompt=task>                             # Prompt used by the shell command\n"
   "\n"
   "# WARNING: Please read the documentation (man task-sync) before setting up\n"
   "#          Taskwarrior for Taskserver synchronization.\n"
-  "#taskd.certificate <certificat file>\n"
+  "#taskd.ca <certificate file>\n"
+  "#taskd.certificate <certificate file>\n"
   "#taskd.credentials <organization>/<name>/<password>\n"
   "#taskd.server      <server>:<port>\n"
+  "taskd.trust=strict\n"
+  "#taskd.trust=ignore hostname\n"
+  "#taskd.trust=allow all\n"
   "taskd.ciphers=NORMAL\n"
-	"\n"
+  "\n"
   "# Aliases - alternate names for commands\n"
   "alias.rm=delete                                # Alias for the delete command\n"
   "alias.history=history.monthly                  # Prefer monthly over annual history reports\n"
   "alias.ghistory=ghistory.monthly                # Prefer monthly graphical over annual history reports\n"
-  "alias._query=export                            # _query is now export\n"
   "alias.burndown=burndown.weekly                 # Prefer the weekly burndown chart\n"
   "alias.shell=exec tasksh                        # Alias old shell command to new shell\n"
   "\n"
   "# Reports\n"
   "\n"
-  "report.long.description=Lists all pending tasks\n"
-  "report.long.columns=id,project,priority,entry,start,due,recur,due.countdown,entry.age,depends,tags,description\n"
-  "report.long.labels=ID,Proj,Pri,Added,Started,Due,Recur,Countdown,Age,Deps,Tags,Description\n"
-  "report.long.sort=due+,priority-,project+\n"
+  "report.long.description=All details of tasks\n"
+  "report.long.labels=ID,A,Created,Mod,Deps,P,Project,Tags,Recur,Wait,Sched,Due,Until,Description\n"
+  "report.long.columns=id,start.active,entry,modified.age,depends,priority,project,tags,recur,wait.remaining,scheduled,due,until,description\n"
   "report.long.filter=status:pending\n"
+  "report.long.sort=modified-\n"
   "\n"
-  "report.list.description=Lists all pending tasks\n"
-  "report.list.columns=id,project,priority,due,start.active,entry.age,description\n"
-  "report.list.labels=ID,Proj,Pri,Due,Active,Age,Description\n"
-  "report.list.sort=due+,priority-,start-,project+\n"
+  "report.list.description=Most details of tasks\n"
+  "report.list.labels=ID,Active,Age,D,P,Project,Tags,R,Sch,Due,Until,Description,Urg\n"
+  "report.list.columns=id,start.age,entry.age,depends.indicator,priority,project,tags,recur.indicator,scheduled.countdown,due,until.remaining,description.count,urgency\n"
   "report.list.filter=status:pending\n"
+  "report.list.sort=start-,due+,project+,urgency-\n"
   "\n"
-  "report.ls.description=Minimal listing of all pending tasks\n"
-  "report.ls.columns=id,project,priority,description\n"
-  "report.ls.labels=ID,Proj,Pri,Description\n"
-  "report.ls.sort=priority-,project+\n"
+  "report.ls.description=Few details of tasks\n"
+  "report.ls.labels=ID,A,D,Project,Tags,R,Wait,S,Due,Until,Description\n"
+  "report.ls.columns=id,start.active,depends.indicator,project,tags,recur.indicator,wait.remaining,scheduled.countdown,due.countdown,until.countdown,description.count\n"
   "report.ls.filter=status:pending\n"
+  "report.ls.sort=start-,description+\n"
   "\n"
-  "report.minimal.description=Minimal listing of all pending tasks\n"
-  "report.minimal.columns=id,project,description.truncated\n"
-  "report.minimal.labels=ID,Proj,Description\n"
-  "report.minimal.sort=project+,description+\n"
-  "report.minimal.filter=status:pending\n"
+  "report.minimal.description=Minimal details of tasks\n"
+  "report.minimal.labels=ID,Project,Tags,Description\n"
+  "report.minimal.columns=id,project,tags.count,description.count\n"
+  "report.minimal.filter=(status:pending or status:waiting)\n"
+  "report.minimal.sort=project+/,description+\n"
   "\n"
-  "report.newest.description=Shows the newest tasks\n"
-  "report.newest.columns=id,project,priority,due,start.active,entry.age,description\n"
-  "report.newest.labels=ID,Proj,Pri,Due,Active,Age,Description\n"
-  "report.newest.sort=id-\n"
-  "report.newest.filter=status:pending limit:10\n"
+  "report.newest.description=Newest tasks\n"
+  "report.newest.labels=ID,Active,Created,Age,Mod,D,P,Project,Tags,R,Wait,Sch,Due,Until,Description\n"
+  "report.newest.columns=id,start.age,entry,entry.age,modified.age,depends.indicator,priority,project,tags,recur.indicator,wait.remaining,scheduled.countdown,due,until.age,description\n"
+  "report.newest.filter=(status:pending or status:waiting)\n"
+  "report.newest.sort=entry-\n"
   "\n"
-  "report.oldest.description=Shows the oldest tasks\n"
-  "report.oldest.columns=id,project,priority,due,start.active,entry.age,description\n"
-  "report.oldest.labels=ID,Proj,Pri,Due,Active,Age,Description\n"
-  "report.oldest.sort=id+\n"
-  "report.oldest.filter=status:pending limit:10\n"
+  "report.oldest.description=Oldest tasks\n"
+  "report.oldest.labels=ID,Active,Created,Age,Mod,D,P,Project,Tags,R,Wait,Sch,Due,Until,Description\n"
+  "report.oldest.columns=id,start.age,entry,entry.age,modified.age,depends.indicator,priority,project,tags,recur.indicator,wait.remaining,scheduled.countdown,due,until.age,description\n"
+  "report.oldest.filter=(status:pending or status:waiting)\n"
+  "report.oldest.sort=entry+\n"
   "\n"
-  "report.overdue.description=Lists overdue tasks\n"
-  "report.overdue.columns=id,project,priority,due,start.active,entry.age,description\n"
-  "report.overdue.labels=ID,Proj,Pri,Due,Active,Age,Description\n"
-  "report.overdue.sort=due+,priority-,start-,project+\n"
-  "report.overdue.filter=status:pending +OVERDUE\n"
+  "report.overdue.description=Overdue tasks\n"
+  "report.overdue.labels=ID,Active,Age,Deps,P,Project,Tag,R,S,Due,Until,Description,Urg\n"
+  "report.overdue.columns=id,start.age,entry.age,depends,priority,project,tags,recur.indicator,scheduled.countdown,due,until,description,urgency\n"
+  "report.overdue.filter=(status:pending or status:waiting) and +OVERDUE\n"
+  "report.overdue.sort=urgency-,due+\n"
   "\n"
-  "report.active.description=Lists active tasks\n"
-  "report.active.columns=id,project,priority,due,start.active,entry.age,description\n"
-  "report.active.labels=ID,Proj,Pri,Due,Active,Age,Description\n"
-  "report.active.sort=due+,priority-,project+\n"
-  "report.active.filter=status:pending +ACTIVE\n"
+  "report.active.description=Active tasks\n"
+  "report.active.labels=ID,Started,Active,Age,D,P,Project,Tags,Recur,W,Sch,Due,Until,Description\n"
+  "report.active.columns=id,start,start.age,entry.age,depends.indicator,priority,project,tags,recur,wait.indicator,scheduled.remaining,due,until,description\n"
+  "report.active.filter=status:pending and +ACTIVE\n"
+  "report.active.sort=project+,start+\n"
   "\n"
-  "report.completed.description=Lists completed tasks\n"
-  "report.completed.columns=end,project,priority,entry.age,description,uuid\n"
-  "report.completed.labels=Complete,Proj,Pri,Age,Description,UUID\n"
-  "report.completed.sort=end+,priority-,project+\n"
+  "report.completed.description=Completed tasks\n"
+  "report.completed.labels=ID,UUID,Created,Completed,took,Deps,P,Project,Tags,R,Due,Description\n"
+  "report.completed.columns=id,uuid.short,entry,end,entry.age,depends,priority,project,tags,recur.indicator,due,description\n"
   "report.completed.filter=status:completed\n"
+  "report.completed.sort=end+\n"
   "\n"
-  "report.recurring.description=Lists recurring tasks\n"
-  "report.recurring.columns=id,project,priority,due,recur,start.active,entry.age,description\n"
-  "report.recurring.labels=ID,Proj,Pri,Due,Recur,Active,Age,Description\n"
-  "report.recurring.sort=due+,priority-,start-,project+\n"
-  "report.recurring.filter=status:pending +CHILD\n"
+  "report.recurring.description=Recurring Tasks\n"
+  "report.recurring.labels=ID,Active,Age,D,P,Project,Tags,Recur,Sch,Due,Until,Description,Urg\n"
+  "report.recurring.columns=id,start.age,entry.age,depends.indicator,priority,project,tags,recur,scheduled.countdown,due,until.remaining,description,urgency\n"
+  "report.recurring.filter=(status:pending or status:waiting) and (+PARENT or +CHILD)\n"
+  "report.recurring.sort=due+,urgency-,entry+\n"
   "\n"
-  "report.waiting.description=Lists all waiting tasks\n"
-  "report.waiting.columns=id,project,priority,wait,entry.age,description\n"
-  "report.waiting.labels=ID,Proj,Pri,Wait,Age,Description\n"
-  "report.waiting.sort=wait+,priority-,project+\n"
-  "report.waiting.filter=status:waiting +WAITING\n"
+  "report.waiting.description=Waiting (hidden) tasks\n"
+  "report.waiting.labels=ID,A,Age,D,P,Project,Tags,R,Wait,for,Sched,Due,Until,Description\n"
+  "report.waiting.columns=id,start.active,entry.age,depends.indicator,priority,project,tags,recur.indicator,wait,wait.remaining,scheduled,due,until,description\n"
+  "report.waiting.filter=+WAITING\n"
+  "report.waiting.sort=due+,wait+,entry+\n"
   "\n"
-  "report.all.description=Lists all pending and completed tasks\n"
-  "report.all.columns=id,status,project,priority,due,end,start.active,entry.age,description\n"
-  "report.all.labels=ID,Status,Proj,Pri,Due,Completed,Active,Age,Description\n"
-  "report.all.sort=entry+\n"
-  "report.all.filter=status.not:deleted\n"
+  "report.all.description=All tasks\n"
+  "report.all.labels=ID,St,UUID,A,Age,Done,D,P,Project,Tags,R,Wait,Sch,Due,Until,Description\n"
+  "report.all.columns=id,status.short,uuid.short,start.active,entry.age,end.age,depends.indicator,priority,project.parent,tags.count,recur.indicator,wait.remaining,scheduled.remaining,due,until.remaining,description\n"
+  "report.all.sort=entry-\n"
   "\n"
-  "report.next.description=Lists the most urgent tasks\n"
-  "report.next.columns=id,project,priority,due,start.active,entry.age,urgency,description\n"
-  "report.next.labels=ID,Proj,Pri,Due,A,Age,Urg,Description\n"
-  "report.next.sort=urgency-,due+,priority-,start-,project+\n"
+  "report.next.description=Most urgent tasks\n"
+  "report.next.labels=ID,Active,Age,Deps,P,Project,Tag,Recur,S,Due,Until,Description,Urg\n"
+  "report.next.columns=id,start.age,entry.age,depends,priority,project,tags,recur,scheduled.countdown,due.remaining,until.remaining,description,urgency\n"
   "report.next.filter=status:pending limit:page\n"
+  "report.next.sort=urgency-\n"
   "\n"
-  "report.ready.description=Lists the most urgent tasks\n"
-  "report.ready.columns=id,project,priority,due,start.active,entry.age,urgency,description\n"
-  "report.ready.labels=ID,Proj,Pri,Due,A,Age,Urg,Description\n"
-  "report.ready.sort=urgency-,due+,priority-,start-,project+\n"
-  "report.ready.filter=status:pending limit:page -WAITING '(scheduled.none: or scheduled.before:now )'\n"
+  "report.ready.description=Most urgent actionable tasks\n"
+  "report.ready.labels=ID,Active,Age,D,P,Project,Tags,R,S,Due,Until,Description,Urg\n"
+  "report.ready.columns=id,start.age,entry.age,depends.indicator,priority,project,tags,recur.indicator,scheduled.countdown,due.countdown,until.remaining,description,urgency\n"
+  "report.ready.filter=+READY\n"
+  "report.ready.sort=start-,urgency-\n"
   "\n"
-  "report.blocked.description=Lists all blocked tasks\n"
+  "report.blocked.description=Blocked tasks\n"
   "report.blocked.columns=id,depends,project,priority,due,start.active,entry.age,description\n"
   "report.blocked.labels=ID,Deps,Proj,Pri,Due,Active,Age,Description\n"
   "report.blocked.sort=due+,priority-,start-,project+\n"
   "report.blocked.filter=status:pending +BLOCKED\n"
   "\n"
-  "report.unblocked.description=Lists all unblocked tasks\n"
+  "report.unblocked.description=Unblocked tasks\n"
   "report.unblocked.columns=id,depends,project,priority,due,start.active,entry.age,description\n"
   "report.unblocked.labels=ID,Deps,Proj,Pri,Due,Active,Age,Description\n"
   "report.unblocked.sort=due+,priority-,start-,project+\n"
   "report.unblocked.filter=status:pending -BLOCKED\n"
   "\n"
-  "report.blocking.description=Lists all blocking tasks\n"
-  "report.blocking.columns=id,depends,project,priority,due,start.active,entry.age,description\n"
-  "report.blocking.labels=ID,Deps,Proj,Pri,Due,Active,Age,Description\n"
-  "report.blocking.sort=due+,priority-,start-,project+\n"
-  "report.blocking.filter=status:pending +BLOCKING\n"
+  "report.blocking.description=Blocking tasks\n"
+  "report.blocking.labels=ID,UUID,A,Deps,Project,Tags,R,W,Sch,Due,Until,Description,Urg\n"
+  "report.blocking.columns=id,uuid.short,start.active,depends,project,tags,recur,wait.indicator,scheduled.remaining,due.remaining,until.remaining,description.count,urgency\n"
+  "report.blocking.sort=urgency-,due+,entry+\n"
+  "report.blocking.filter= status:pending +BLOCKING\n"
   "\n";
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -549,6 +600,10 @@ void Config::createDefaultRC (const std::string& rc, const std::string& data)
            << "#include " << TASK_RCDIR << "/dark-violets-256.theme\n"
            << "#include " << TASK_RCDIR << "/dark-yellow-green.theme\n"
            << "#include " << TASK_RCDIR << "/dark-gray-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-gray-blue-256.theme\n"
+           << "#include " << TASK_RCDIR << "/solarized-dark-256.theme\n"
+           << "#include " << TASK_RCDIR << "/solarized-light-256.theme\n"
+           << "#include " << TASK_RCDIR << "/no-color.theme\n"
            << "\n";
 
   // Write out the new file.
@@ -566,6 +621,9 @@ void Config::createDefaultData (const std::string& data)
       throw std::string ("Error: rc.data.location does not exist - exiting according to rc.exit.on.missing.db setting.");
 
     d.create ();
+
+    d += "hooks";
+    d.create ();
   }
 }
 
@@ -581,6 +639,12 @@ void Config::clear ()
   std::map <std::string, std::string>::clear ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const bool Config::has (const std::string& key)
+{
+  return (*this).find (key) != (*this).end ();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Return the configuration value given the specified key.
 const std::string Config::get (const std::string& key)
