@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2014, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +36,8 @@
 #include <Context.h>
 #include <Directory.h>
 #include <File.h>
+#include <Eval.h>
+#include <Variant.h>
 #include <text.h>
 #include <util.h>
 #include <main.h>
@@ -45,16 +46,35 @@
 #include <commit.h>
 #endif
 
+// Supported modifiers, synonyms on the same line.
+static const char* modifierNames[] =
+{
+  "before",     "under",    "below",
+  "after",      "over",     "above",
+  "none",
+  "any",
+  "is",         "equals",
+  "isnt",       "not",
+  "has",        "contains",
+  "hasnt",
+  "startswith", "left",
+  "endswith",   "right",
+  "word",
+  "noword"
+};
+
+#define NUM_MODIFIER_NAMES       (sizeof (modifierNames) / sizeof (modifierNames[0]))
+
 ////////////////////////////////////////////////////////////////////////////////
 Context::Context ()
-: rc_file ()
-, data_dir ()
-, extension_dir ()
+: rc_file ("~/.taskrc")
+, data_dir ("~/.task")
 , config ()
 , tdb2 ()
 , dom ()
 , determine_color_use (true)
 , use_color (true)
+, run_gc (true)
 , verbosity_legacy (false)
 , terminal_width (0)
 , terminal_height (0)
@@ -81,22 +101,19 @@ int Context::initialize (int argc, const char** argv)
 
   try
   {
-    // char** argv --> std::vector <std::string> Context::a3.
-    a3.capture (argc, argv);
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [1] Load the correct config file.
+    //     - Default to ~/.taskrc (ctor).
+    //     - Allow command line override rc:<file>
+    //     - Allow $TASKRC override.
+    //     - Load resultant file.
+    //     - Apply command line overrides to the config.
+    //
+    ////////////////////////////////////////////////////////////////////////////
 
-    // echo one two -- three | task zero --> task zero one two
-    // 'three' is left in the input buffer.
-    a3.append_stdin ();
+    CLI::getOverride (argc, argv, home_dir, rc_file);
 
-    // Assume default .taskrc and .task locations.
-    assumeLocations ();
-
-    // Process 'rc:<file>' command line override, and remove the argument from the
-    // Context::a3.
-    a3.categorize ();
-    a3.rc_override (home_dir, rc_file);
-
-    // TASKRC environment variable overrides the command line.
     char* override = getenv ("TASKRC");
     if (override)
     {
@@ -104,16 +121,22 @@ int Context::initialize (int argc, const char** argv)
       header (format (STRING_CONTEXT_RC_OVERRIDE, rc_file._data));
     }
 
-    // Dump any existing values and load rc file.
     config.clear ();
-    config.load  (rc_file);
+    config.load (rc_file);
+    CLI::applyOverrides (argc, argv);
 
-    // The data location, Context::data_dir, is determined from the assumed
-    // location (~/.task), or set by data.location in the config file, or
-    // overridden by rc.data.location on the command line.
-    std::string location;
-    a3.get_data_location (location);
-    data_dir = Directory (location);
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [2] Locate the data directory.
+    //     - Default to ~/.task (ctor).
+    //     - Allow command line override rc.data.location:<dir>
+    //     - Allow $TASKDATA override.
+    //     - Inform TDB2 where to find data.
+    //     - Create the rc_file and data_dir, if necessary.
+    //
+    ////////////////////////////////////////////////////////////////////////////
+
+    CLI::getDataLocation (argc, argv, data_dir);
 
     override = getenv ("TASKDATA");
     if (override)
@@ -123,68 +146,114 @@ int Context::initialize (int argc, const char** argv)
       header (format (STRING_CONTEXT_DATA_OVERRIDE, data_dir._data));
     }
 
-/* TODO Enable this when the time is right, say for 2.1
-    extension_dir = data_dir._data + "/extensions";
-*/
-
-    // Create missing config file and data directory, if necessary.
-    a3.apply_overrides ();
+    tdb2.set_location (data_dir);
     createDefaultConfig ();
 
-    // Handle Aliases.
-    loadAliases ();
-    a3.resolve_aliases ();
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [3] Instantiate Command objects and capture entities.
+    //
+    ////////////////////////////////////////////////////////////////////////////
 
-    // Apply rc overrides to Context::config, capturing raw args for later use.
-    a3.apply_overrides ();
-
-    // Initialize the color rules, if necessary.
-    if (color ())
-      initializeColorRules ();
-
-    // Instantiate built-in command objects.
     Command::factory (commands);
+    std::map <std::string, Command*>::iterator cmd;
+    for (cmd = commands.begin (); cmd != commands.end (); ++cmd)
+    {
+      cli.entity ("cmd", cmd->first);
+      cli.entity ((cmd->second->read_only () ? "readcmd" : "writecmd"), cmd->first);
+
+      if (cmd->first[0] == '_')
+        cli.entity ("helper", cmd->first);
+    }
+
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [4] Instantiate Column objects and capture entities.
+    //
+    ////////////////////////////////////////////////////////////////////////////
 
-    // Instantiate built-in column objects.
     Column::factory (columns);
+    std::map <std::string, Column*>::iterator col;
+    for (col = columns.begin (); col != columns.end (); ++col)
+      cli.entity ("attribute", col->first);
 
-    // Static initialization to decouple code.
+    cli.entity ("pseudo", "limit");
+
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [5] Capture modifier and operator entities.
+    //
+    ////////////////////////////////////////////////////////////////////////////
+
+    for (unsigned int i = 0; i < NUM_MODIFIER_NAMES; ++i)
+      cli.entity ("modifier", modifierNames[i]);
+
+    std::vector <std::string> operators;
+    Eval::getOperators (operators);
+    std::vector <std::string>::iterator op;
+    for (op = operators.begin (); op != operators.end (); ++op)
+      cli.entity ("operator", *op);
+
+    Eval::getBinaryOperators (operators);
+    for (op = operators.begin (); op != operators.end (); ++op)
+      cli.entity ("binary_operator", *op);
+
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [6] Complete the Context initialization.
+    //
+    ////////////////////////////////////////////////////////////////////////////
+
+    initializeColorRules ();
     staticInitialization ();
+    propagateDebug ();
+    loadAliases ();
 
-    // Categorize all arguments one more time.  THIS IS NECESSARY - it helps the
-    // following inject_defaults method determine whether there needs to be a
-    // default command assumed.
-    a3.categorize ();
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [7] Parse the command line.
+    //
+    ////////////////////////////////////////////////////////////////////////////
 
-    // Handle default command and assumed 'info' command.
-    a3.inject_defaults ();
+    // Scan command line for 'rc:<file>' only.
+    cli.initialize (argc, argv);
+    cli.analyze (true, true);
 
-    // The re-categorization allows all injected arguments to be properly given
-    // a category.
-    a3.categorize ();
-    a3.dump ("Context::initialize");
+    // Extract a recomposed command line.
+    bool foundDefault = false;
+    bool foundAssumed = false;
+    std::string combined;
+    std::vector <A>::const_iterator a;
+    for (a = cli._args.begin (); a != cli._args.end (); ++a)
+    {
+      if (combined.length ())
+        combined += ' ';
 
-    // TODO Instantiate extension command objects.
-    // TODO Instantiate default command object.
+      if (a->attribute ("canonical") != "")
+        combined += a->attribute ("canonical");
+      else
+        combined += a->attribute ("raw");
 
-    // TODO Instantiate extension column objects.
+      if (a->hasTag ("DEFAULT"))
+        foundDefault = true;
 
-    // TODO Instantiate extension UDA objects.
-    // TODO Instantiate extension format objects.
+      if (a->hasTag ("ASSUMED"))
+        foundAssumed = true;
+    }
 
-    // If there is a locale variant (en-US.<variant>), then strip it.
-    std::string locale = config.get ("locale");
-    std::string::size_type period = locale.find ('.');
-    if (period != std::string::npos)
-      locale = locale.substr (0, period);
+    if (foundDefault)
+      header ("[" + combined + "]");
 
-    // Initialize the database.
-    tdb2.set_location (data_dir);
+    if (foundAssumed)
+      header (STRING_ASSUME_INFO);
+
+    ////////////////////////////////////////////////////////////////////////////
+    //
+    // [8] Initialize hooks.
+    //
+    ////////////////////////////////////////////////////////////////////////////
 
-    // Hook system init, plus post-start event occurring at the first possible
-    // moment after hook initialization.
     hooks.initialize ();
-    hooks.trigger ("on-launch");
   }
 
   catch (const std::string& message)
@@ -193,49 +262,56 @@ int Context::initialize (int argc, const char** argv)
     rc = 2;
   }
 
+  catch (int)
+  {
+    // Hooks can terminate processing by throwing integers.
+    rc = 4;
+  }
+
   catch (...)
   {
     error (STRING_UNKNOWN_ERROR);
     rc = 3;
   }
 
-  // Dump all debug messages, controlled by rc.debug.
-  if (rc && config.getBoolean ("debug"))
-  {
-    std::vector <std::string>::iterator d;
-    for (d = debugMessages.begin (); d != debugMessages.end (); ++d)
-      if (color ())
-        std::cerr << colorizeDebug (*d) << "\n";
-      else
-        std::cerr << *d << "\n";
-  }
-
-  // Dump all headers, controlled by 'header' verbosity token.
-  if (rc && verbose ("header"))
-  {
-    std::vector <std::string>::iterator h;
-    for (h = headers.begin (); h != headers.end (); ++h)
-      if (color ())
-        std::cerr << colorizeHeader (*h) << "\n";
-      else
-        std::cerr << *h << "\n";
-  }
-
-  // Dump all footnotes, controlled by 'footnote' verbosity token.
-  if (rc && verbose ("footnote"))
-  {
-    std::vector <std::string>::iterator f;
-    for (f = footnotes.begin (); f != footnotes.end (); ++f)
-      if (color ())
-        std::cerr << colorizeFootnote (*f) << "\n";
-      else
-        std::cerr << *f << "\n";
-  }
-
-  // Dump all errors, non-maskable.
-  // Colorized as footnotes.
+  // On initialization failure...
   if (rc)
   {
+    // Dump all debug messages, controlled by rc.debug.
+    if (config.getBoolean ("debug"))
+    {
+      std::vector <std::string>::iterator d;
+      for (d = debugMessages.begin (); d != debugMessages.end (); ++d)
+        if (color ())
+          std::cerr << colorizeDebug (*d) << "\n";
+        else
+          std::cerr << *d << "\n";
+    }
+
+    // Dump all headers, controlled by 'header' verbosity token.
+    if (verbose ("header"))
+    {
+      std::vector <std::string>::iterator h;
+      for (h = headers.begin (); h != headers.end (); ++h)
+        if (color ())
+          std::cerr << colorizeHeader (*h) << "\n";
+        else
+          std::cerr << *h << "\n";
+    }
+
+    // Dump all footnotes, controlled by 'footnote' verbosity token.
+    if (verbose ("footnote"))
+    {
+      std::vector <std::string>::iterator f;
+      for (f = footnotes.begin (); f != footnotes.end (); ++f)
+        if (color ())
+          std::cerr << colorizeFootnote (*f) << "\n";
+        else
+          std::cerr << *f << "\n";
+    }
+
+    // Dump all errors, non-maskable.
+    // Colorized as footnotes.
     std::vector <std::string>::iterator e;
     for (e = errors.begin (); e != errors.end (); ++e)
       if (color ())
@@ -256,7 +332,10 @@ int Context::run ()
 
   try
   {
+    hooks.onLaunch ();
     rc = dispatch (output);
+    tdb2.commit ();           // Harmless if called when nothing changed.
+    hooks.onExit ();          // No chance to update data.
 
     std::stringstream s;
     s << "Perf "
@@ -277,13 +356,15 @@ int Context::run ()
       << " commit:" << timer_commit.total ()
       << " sort:"   << timer_sort.total ()
       << " render:" << timer_render.total ()
+      << " hooks:"  << timer_hooks.total ()
       << " total:"  << (timer_init.total ()   +
                         timer_load.total ()   +
                         timer_gc.total ()     +
                         timer_filter.total () +
                         timer_commit.total () +
                         timer_sort.total ()   +
-                        timer_render.total ())
+                        timer_render.total () +
+                        timer_hooks.total ())
       << "\n";
     debug (s.str ());
   }
@@ -294,6 +375,12 @@ int Context::run ()
     rc = 2;
   }
 
+  catch (int)
+  {
+    // Hooks can terminate processing by throwing integers.
+    rc = 4;
+  }
+
   catch (...)
   {
     error (STRING_UNKNOWN_ERROR);
@@ -345,56 +432,47 @@ int Context::run ()
     else
       std::cerr << *e << "\n";
 
-  hooks.trigger ("on-exit");
   return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Locate and dispatch to the command whose keyword matches via autoComplete
-// with the earliest argument.
+// Dispatch to the command found by the CLI parser.
 int Context::dispatch (std::string &out)
 {
   // Autocomplete args against keywords.
-  std::string command;
-  if (a3.find_command (command))
+  std::string command = cli.getCommand ();
+  if (command != "")
   {
     updateXtermTitle ();
-
     updateVerbosity ();
 
     Command* c = commands[command];
+    assert (c);
 
     // GC is invoked prior to running any command that displays task IDs, if
     // possible.
-    bool auto_commit = false;
     if (c->displays_id () && !tdb2.read_only ())
     {
+      run_gc = true;
       tdb2.gc ();
-      auto_commit = true;
+    }
+    else
+    {
+      run_gc = false;
     }
 
+/*
     // Only read-only commands can be run when TDB2 is read-only.
     // TODO Implement TDB2::read_only
-/*
     if (tdb2.read_only () && !c->read_only ())
       throw std::string ("");
 */
 
-    int rc = c->execute (out);
-    if (auto_commit)
-      tdb2.commit ();
-
-    // Write commands cause an update of the shadow file, if configured.
-    if (! c->read_only ())
-      shadow ();
-
-    return rc;
+    return c->execute (out);
   }
 
   assert (commands["help"]);
   return commands["help"]->execute (out);
-
-  return 1;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -407,7 +485,7 @@ bool Context::color ()
     use_color = config.getBoolean ("color");
 
     // Only tty's support color.
-    if (! isatty (fileno (stdout)))
+    if (! isatty (STDOUT_FILENO))
     {
       // No ioctl.
       config.set ("detection", "off");
@@ -426,8 +504,6 @@ bool Context::color ()
 
     // No need to go through this again.
     determine_color_use = false;
-
-    debug ("Context::color --> " + std::string (use_color ? "on" : "off"));
   }
 
   // Cached result.
@@ -456,19 +532,24 @@ bool Context::verbose (const std::string& token)
     split (verbosity, config.get ("verbose"), ',');
 
     // Regular feedback means almost everything.
+    // This odd test is to see if a Boolean-false value is a real one, which
+    // means it is not 1/true/T/yes/on, but also should not be one of the
+    // valid tokens either.
     if (!verbosity_legacy               &&
-        verbosity.size () == 1          &&
+        verbosity.size ()               &&
         verbosity[0]      != "nothing"  &&
         verbosity[0]      != "blank"    &&  // This list must be complete.
         verbosity[0]      != "header"   &&  //
         verbosity[0]      != "footnote" &&  //
         verbosity[0]      != "label"    &&  //
         verbosity[0]      != "new-id"   &&  //
+        verbosity[0]      != "new-uuid" &&  //
         verbosity[0]      != "affected" &&  //
         verbosity[0]      != "edit"     &&  //
         verbosity[0]      != "special"  &&  //
         verbosity[0]      != "project"  &&  //
-        verbosity[0]      != "sync")        //
+        verbosity[0]      != "sync"     &&  //
+        verbosity[0]      != "filter")      //
     {
       verbosity.clear ();
 
@@ -485,7 +566,8 @@ bool Context::verbose (const std::string& token)
     return true;
 
   // rc.verbose=nothing overrides all.
-  if (verbosity[0] == "nothing")
+  if (verbosity.size () == 1 &&
+      verbosity[0] == "nothing")
     return false;
 
   // Specific token match.
@@ -495,59 +577,6 @@ bool Context::verbose (const std::string& token)
   return false;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// This needs to be taken out and shot, as soon as hooks will allow.
-void Context::shadow ()
-{
-  std::string file_name = config.get ("shadow.file");
-  std::string command   = config.get ("shadow.command");
-  std::string rcfile    = rc_file;
-
-  // A missing shadow file command uses the default command instead.
-  if (command == "")
-    command = config.get ("default.command");
-
-  if (file_name != "" &&
-      command != "")
-  {
-    File shadow_file (file_name);
-
-    // Check for dangerous shadow file settings.
-    std::string location = config.get ("data.location");
-    if (shadow_file._data == location + "/pending.data")
-      throw std::string (STRING_CONTEXT_SHADOW_P);
-
-    if (shadow_file._data == location + "/completed.data")
-      throw std::string (STRING_CONTEXT_SHADOW_C);
-
-    if (shadow_file._data == location + "/undo.data")
-      throw std::string (STRING_CONTEXT_SHADOW_U);
-
-    if (shadow_file._data == location + "/backlog.data")
-      throw std::string (STRING_CONTEXT_SHADOW_B);
-
-    // Compose the command.  Put the rc overrides up front, so that they may
-    // be overridden by rc.shadow.command.
-    command = program               +
-              " rc.detection:off"   +  // No need to determine terminal size
-              " rc.color:off"       +  // Color off by default
-              " rc.gc:off "         +  // GC off, to reduce headaches
-              " rc.locking:off"     +  // No file locking
-              " rc:" + rcfile + " " +  // Use specified rc file
-              command               +  // User specified command
-              " >"                  +  // Capture
-              shadow_file._data;       // User specified file
-
-    debug ("Running shadow command: " + command);
-    system (command.c_str ());
-
-    // Optionally display a notification that the shadow file was updated.
-    // TODO Convert to a verbosity token.
-    if (config.getBoolean ("shadow.notify"))
-      footnote (format (STRING_CONTEXT_SHADOW_UPDATE, shadow_file._data));
-  }
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 const std::vector <std::string> Context::getColumns () const
 {
@@ -570,27 +599,72 @@ const std::vector <std::string> Context::getCommands () const
   return output;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// A value of zero mean unlimited.
+// A value of 'page' means however many screen lines there are.
+// A value of a positive integer is a row/task limit.
+void Context::getLimits (int& rows, int& lines)
+{
+  rows = 0;
+  lines = 0;
+
+  // This is an integer specified as a filter (limit:10).
+  std::string limit = cli.getLimit ();
+  if (limit != "")
+  {
+    if (limit == "page")
+    {
+      rows = 0;
+      lines = getHeight ();
+    }
+    else
+    {
+      rows = (int) strtol (limit.c_str (), NULL, 10);
+      lines = 0;
+    }
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // The 'Task' object, among others, is shared between projects.  To make this
 // easier, it has been decoupled from Context.
 void Context::staticInitialization ()
 {
+  CLI::minimumMatchLength   = config.getInteger ("abbreviation.minimum");
+
   Task::defaultProject      = config.get ("default.project");
-  Task::defaultPriority     = config.get ("default.priority");
   Task::defaultDue          = config.get ("default.due");
-  Task::searchCaseSensitive = config.getBoolean ("search.case.sensitive");
-  Task::regex               = config.getBoolean ("regex");
+
+  Task::searchCaseSensitive = Variant::searchCaseSensitive = config.getBoolean ("search.case.sensitive");
+  Task::regex               = Variant::searchUsingRegex    = config.getBoolean ("regex");
+  Lexer::dateFormat         = Variant::dateFormat          = config.get ("dateformat");
+  Lexer::isoEnabled         = Variant::isoEnabled          = config.getBoolean ("date.iso");
+
+  Config::const_iterator rc;
+  for (rc = config.begin (); rc != config.end (); ++rc)
+  {
+    if (rc->first.substr (0, 4) == "uda." &&
+        rc->first.substr (rc->first.length () - 7, 7) == ".values")
+    {
+      std::string name = rc->first.substr (4, rc->first.length () - 7 - 4);
+      std::vector <std::string> values;
+      split (values, rc->second, ',');
+
+      for (auto r = values.rbegin(); r != values.rend (); ++r)
+        Task::customOrder[name].push_back (*r);
+    }
+  }
 
   std::map <std::string, Column*>::iterator i;
   for (i = columns.begin (); i != columns.end (); ++i)
     Task::attributes[i->first] = i->second->type ();
 
-  Task::urgencyPriorityCoefficient    = config.getReal ("urgency.priority.coefficient");
   Task::urgencyProjectCoefficient     = config.getReal ("urgency.project.coefficient");
   Task::urgencyActiveCoefficient      = config.getReal ("urgency.active.coefficient");
   Task::urgencyScheduledCoefficient   = config.getReal ("urgency.scheduled.coefficient");
   Task::urgencyWaitingCoefficient     = config.getReal ("urgency.waiting.coefficient");
   Task::urgencyBlockedCoefficient     = config.getReal ("urgency.blocked.coefficient");
+  Task::urgencyInheritCoefficient     = config.getReal ("urgency.inherit.coefficient");
   Task::urgencyAnnotationsCoefficient = config.getReal ("urgency.annotations.coefficient");
   Task::urgencyTagsCoefficient        = config.getReal ("urgency.tags.coefficient");
   Task::urgencyNextCoefficient        = config.getReal ("urgency.next.coefficient");
@@ -602,21 +676,11 @@ void Context::staticInitialization ()
   // Tag- and project-specific coefficients.
   std::vector <std::string> all;
   config.all (all);
-
   std::vector <std::string>::iterator var;
   for (var = all.begin (); var != all.end (); ++var)
-  {
     if (var->substr (0, 13) == "urgency.user." ||
         var->substr (0, 12) == "urgency.uda.")
       Task::coefficients[*var] = config.getReal (*var);
-  }
-}
-
-////////////////////////////////////////////////////////////////////////////////
-void Context::assumeLocations ()
-{
-  rc_file  = File      ("~/.taskrc");
-  data_dir = Directory ("~/.task");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -634,45 +698,34 @@ void Context::createDefaultConfig ()
 
   // Create data location, if necessary.
   config.createDefaultData (data_dir);
-
-  // Create extension directory, if necessary.
-/* TODO Enable this when the time is right, say for 2.4
-  if (! extension_dir.exists ())
-    extension_dir.create ();
-*/
-}
-
-////////////////////////////////////////////////////////////////////////////////
-void Context::loadAliases ()
-{
-  aliases.clear ();
-
-  std::vector <std::string> vars;
-  config.all (vars);
-
-  std::vector <std::string>::iterator var;
-  for (var = vars.begin (); var != vars.end (); ++var)
-    if (var->substr (0, 6) == "alias.")
-      aliases[var->substr (6)] = config.get (*var);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Context::decomposeSortField (
   const std::string& field,
   std::string& key,
-  bool& ascending)
+  bool& ascending,
+  bool& breakIndicator)
 {
   int length = field.length ();
 
-  if (field[length - 1] == '+')
+  int decoration = 1;
+  breakIndicator = false;
+  if (field[length - decoration] == '/')
+  {
+    breakIndicator = true;
+    ++decoration;
+  }
+
+  if (field[length - decoration] == '+')
   {
     ascending = true;
-    key = field.substr (0, length - 1);
+    key = field.substr (0, length - decoration);
   }
-  else if (field[length - 1] == '-')
+  else if (field[length - decoration] == '-')
   {
     ascending = false;
-    key = field.substr (0, length - 1);
+    key = field.substr (0, length - decoration);
   }
   else
   {
@@ -685,10 +738,11 @@ void Context::decomposeSortField (
 // Note: The reason some of these are commented out is because the ::clear
 // method is not really "clear" but "clear_some".  Some members do not need to
 // be initialized.  That makes this method something of a misnomer.  So be it.
+//
+// TODO Is this method used anywhere?
 void Context::clear ()
 {
   tdb2.clear ();
-  a3.clear ();
 
   // Eliminate the command objects.
   std::map <std::string, Command*>::iterator com;
@@ -703,7 +757,6 @@ void Context::clear ()
     delete col->second;
 
   columns.clear ();
-
   clearMessages ();
 }
 
@@ -712,13 +765,20 @@ void Context::clear ()
 // this output?'.
 void Context::updateXtermTitle ()
 {
-  if (config.getBoolean ("xterm.title") && isatty (fileno (stdout)))
+  if (config.getBoolean ("xterm.title") && isatty (STDOUT_FILENO))
   {
-    std::string command;
-    a3.find_command (command);
-
+    std::string command = cli.getCommand ();
     std::string title;
-    join (title, " ", a3.list ());
+
+    std::vector <A>::const_iterator a;
+    for (a = cli._args.begin (); a != cli._args.end (); ++a)
+    {
+      if (a != cli._args.begin ())
+        title += ' ';
+
+      title += a->attribute ("raw");
+    }
+
     std::cout << "]0;task " << command << " " << title << "";
   }
 }
@@ -727,10 +787,9 @@ void Context::updateXtermTitle ()
 // This function allows a clean output if the command is a helper subcommand.
 void Context::updateVerbosity ()
 {
-  std::string command;
-  a3.find_command (command);
-
-  if (command[0] == '_')
+  std::string command = cli.getCommand ();
+  if (command != "" &&
+      command[0] == '_')
   {
     verbosity.clear ();
     verbosity.push_back ("nothing");
@@ -738,6 +797,41 @@ void Context::updateVerbosity ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+void Context::loadAliases ()
+{
+  std::map <std::string, std::string>::iterator i;
+  for (i = config.begin (); i != config.end (); ++i)
+    if (i->first.substr (0, 6) == "alias.")
+      cli.alias (i->first.substr (6), i->second);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Using the general rc.debug setting automaticalls sets debug.tls, debug.hooks
+// and debug.parser, unless they already have values, which by default they do
+// not.
+void Context::propagateDebug ()
+{
+  if (config.getBoolean ("debug"))
+  {
+    if (! config.has ("debug.tls"))
+      config.set ("debug.tls", 2);
+
+    if (! config.has ("debug.hooks"))
+      config.set ("debug.hooks", 1);
+
+    if (! config.has ("debug.parser"))
+      config.set ("debug.parser", 1);
+  }
+  else
+  {
+    if ((config.has ("debug.hooks")  && config.getInteger ("debug.hooks")) ||
+        (config.has ("debug.parser") && config.getInteger ("debug.parser")) )
+      config.set ("debug", true);
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// No duplicates.
 void Context::header (const std::string& input)
 {
   if (input.length () &&
@@ -746,6 +840,7 @@ void Context::header (const std::string& input)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// No duplicates.
 void Context::footnote (const std::string& input)
 {
   if (input.length () &&
@@ -754,6 +849,7 @@ void Context::footnote (const std::string& input)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// No duplicates.
 void Context::error (const std::string& input)
 {
   if (input.length () &&
@@ -764,8 +860,7 @@ void Context::error (const std::string& input)
 ////////////////////////////////////////////////////////////////////////////////
 void Context::debug (const std::string& input)
 {
-  if (input.length () &&
-      std::find (debugMessages.begin (), debugMessages.end (), input) == debugMessages.end ())
+  if (input.length ())
     debugMessages.push_back (input);
 }
 
