@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2014, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -44,22 +43,6 @@
 
 extern Context context;
 
-#define NDEBUG
-#include <assert.h>
-#include <Taskmod.h>
-
-#define DEBUG_OUTPUT 0
-
-#if DEBUG_OUTPUT > 0
-  #define DEBUG_STR(str)       std::cerr << "DEBUG: " << str << "\n"; std::cerr.flush()
-  #define DEBUG_STR_PART(str)  std::cerr << "DEBUG: " << str; std::cerr.flush()
-  #define DEBUG_STR_END(str)   std::cerr << str << "\n"; std::cerr.flush()
-#else
-  #define DEBUG_STR(str)
-  #define DEBUG_STR_PART(str)
-  #define DEBUG_STR_END(str)
-#endif
-
 ////////////////////////////////////////////////////////////////////////////////
 TF2::TF2 ()
 : _read_only (false)
@@ -74,6 +57,9 @@ TF2::TF2 ()
 ////////////////////////////////////////////////////////////////////////////////
 TF2::~TF2 ()
 {
+  if (_dirty && context.verbose ("debug"))
+    std::cout << format (STRING_TDB2_DIRTY_EXIT, _file)
+              << "\n";
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -116,7 +102,7 @@ bool TF2::get (int id, Task& task)
   // pending.data file, the task in question cannot appear earlier than line
   // (id - 1) in the file.  It can, however, appear significantly later because
   // it is not known how recent a GC operation was run.
-  for (int i = id - 1; i < _tasks.size (); ++i)
+  for (unsigned int i = id - 1; i < _tasks.size (); ++i)
   {
     if (_tasks[i].id == id)
     {
@@ -149,16 +135,36 @@ bool TF2::get (const std::string& uuid, Task& task)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TF2::add_task (const Task& task)
+bool TF2::has (const std::string& uuid)
+{
+  if (! _loaded_tasks)
+    load_tasks ();
+
+  std::vector <Task>::iterator i;
+  for (i = _tasks.begin (); i != _tasks.end (); ++i)
+    if (i->get ("uuid") == uuid)
+      return true;
+
+  return false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void TF2::add_task (Task& task)
 {
   _tasks.push_back (task);           // For subsequent queries
   _added_tasks.push_back (task);     // For commit/synch
 
-/* TODO handle 'add' and 'log'.
-  int id = context.tdb2.next_id ();
-  _I2U[id] = task.get ("uuid");
-  _U2I[task.get ("uuid")] = id;
-*/
+  Task::status status = task.getStatus ();
+  if (task.id == 0 &&
+      (status == Task::pending   ||
+       status == Task::recurring ||
+       status == Task::waiting))
+  {
+    task.id = context.tdb2.next_id ();
+  }
+
+  _I2U[task.id] = task.get ("uuid");
+  _U2I[task.get ("uuid")] = task.id;
 
   _dirty = true;
 }
@@ -176,6 +182,7 @@ bool TF2::modify_task (const Task& task)
       *i = task;
       _modified_tasks.push_back (task);
       _dirty = true;
+
       return true;
     }
   }
@@ -250,6 +257,9 @@ void TF2::commit ()
     {
       if (_file.open ())
       {
+        if (context.config.getBoolean ("locking"))
+          _file.waitForLock ();
+
         // Truncate the file and rewrite.
         _file.truncate ();
 
@@ -306,9 +316,15 @@ void TF2::load_tasks ()
       ++line_number;
       Task task (*i);
 
-      // Some tasks gets an ID.
+      // Some tasks get an ID.
       if (_has_ids)
-        task.id = context.tdb2.next_id ();
+      {
+        Task::status status = task.getStatus ();
+        // Completed / deleted tasks in pending.data get an ID if GC is off.
+        if (!context.run_gc ||
+            (status != Task::completed && status != Task::deleted))
+          task.id = context.tdb2.next_id ();
+      }
 
       _tasks.push_back (task);
 
@@ -452,9 +468,13 @@ void TF2::dependency_scan ()
         {
           if (right->get ("uuid") == *d)
           {
-            Task::status status = right->getStatus ();
-            if (status != Task::completed &&
-                status != Task::deleted)
+            // GC hasn't run yet, check both tasks for their current status
+            Task::status lstatus = left->getStatus ();
+            Task::status rstatus = right->getStatus ();
+            if (lstatus != Task::completed &&
+                lstatus != Task::deleted &&
+                rstatus != Task::completed &&
+                rstatus != Task::deleted)
             {
               left->is_blocked = true;
               right->is_blocking = true;
@@ -549,49 +569,63 @@ void TDB2::set_location (const std::string& location)
 void TDB2::add (Task& task, bool add_to_backlog /* = true */)
 {
   // Ensure the task is consistent, and provide defaults if necessary.
-  task.validate ();
+  // bool argument to validate() is "applyDefault". Pass add_to_backlog through
+  // in order to not apply defaults to synchronized tasks.
+  task.validate (add_to_backlog);
+  std::string uuid = task.get ("uuid");
 
   // If the tasks are loaded, then verify that this uuid is not already in
   // the file.
-  if (!verifyUniqueUUID (task.get ("uuid")))
-    throw format (STRING_TDB2_UUID_NOT_UNIQUE, task.get ("uuid"));
+  if (!verifyUniqueUUID (uuid))
+    throw format (STRING_TDB2_UUID_NOT_UNIQUE, uuid);
 
-  // Add new task to either pending or completed.
-  std::string status = task.get ("status");
-  if (status == "completed" ||
-      status == "deleted")
-    completed.add_task (task);
-  else
-    pending.add_task (task);
-
-  // Add undo data lines:
-  //   time <time>
-  //   new <task>
-  //   ---
-  undo.add_line ("time " + Date ().toEpochString () + "\n");
-  undo.add_line ("new " + task.composeF4 () + "\n");
-  undo.add_line ("---\n");
-
-  // Add task to backlog.
+  // Only locally-added tasks trigger hooks.  This means that tasks introduced
+  // via 'sync' do not trigger hooks.
   if (add_to_backlog)
-    backlog.add_line (task.composeJSON () + "\n");
+    context.hooks.onAdd (task);
+
+  update (uuid, task, add_to_backlog, true);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void TDB2::modify (Task& task, bool add_to_backlog /* = true */)
 {
-  // Ensure the task is consistent, and provide defaults if necessary.
+  // Ensure the task is consistent.
+  task.validate (false);
+  std::string uuid = task.get ("uuid");
+
+  // Get the unmodified task as reference, so the hook can compare.
+  if (add_to_backlog)
+  {
+    Task original;
+    get (uuid, original);
+    context.hooks.onModify (original, task);
+  }
+
+  update (uuid, task, add_to_backlog);
+}
+
+////////////////////////////////////////////////////////////////////////////////
+void TDB2::update (
+  const std::string& uuid,
+  Task& task,
+  const bool add_to_backlog,
+  const bool addition /* = false */)
+{
+  // Validate to add metadata.
   task.validate (false);
 
-  // All modified tasks are timestamped.
-  task.setModified ();
-
-  // Find task, overwrite it.
+  // If the task already exists, it is a modification, else addition.
   Task original;
-  get (task.get ("uuid"), original);
-
-  if (taskDiff (original, task))
+  if (not addition && get (task.get ("uuid"), original))
   {
+    if (add_to_backlog)
+    {
+      // All locally modified tasks are timestamped, implicitly overwriting any
+      // changes the user or hooks tried to apply to the "modified" attribute.
+      task.setAsNow ("modified");
+    }
+
     // Update the task, wherever it is.
     if (!pending.modify_task (task))
       completed.modify_task (task);
@@ -604,11 +638,29 @@ void TDB2::modify (Task& task, bool add_to_backlog /* = true */)
     undo.add_line ("old " + original.composeF4 () + "\n");
     undo.add_line ("new " + task.composeF4 () + "\n");
     undo.add_line ("---\n");
-
-    // Add modified task to backlog.
-    if (add_to_backlog)
-      backlog.add_line (task.composeJSON () + "\n");
   }
+  else
+  {
+    // Add new task to either pending or completed.
+    std::string status = task.get ("status");
+    if (status == "completed" ||
+        status == "deleted")
+      completed.add_task (task);
+    else
+      pending.add_task (task);
+
+    // Add undo data lines:
+    //   time <time>
+    //   new <task>
+    //   ---
+    undo.add_line ("time " + Date ().toEpochString () + "\n");
+    undo.add_line ("new " + task.composeF4 () + "\n");
+    undo.add_line ("---\n");
+  }
+
+  // Add task to backlog.
+  if (add_to_backlog)
+    backlog.add_line (task.composeJSON () + "\n");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -626,6 +678,8 @@ void TDB2::commit ()
   dump ();
   context.timer_commit.start ();
 
+  gather_changes ();
+
   pending.commit ();
   completed.commit ();
   undo.commit ();
@@ -644,634 +698,27 @@ void TDB2::commit ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Helper function for TDB::merge
-void readTaskmods (std::vector <std::string> &input,
-                   std::vector <std::string>::iterator &start,
-                   std::list<Taskmod> &list)
+void TDB2::gather_changes ()
 {
-  static int   resourceID = 1;
-  std::string  line;
-  Taskmod      tmod_tmp(resourceID++);
+  _changes.clear ();
+  std::vector <Task>::iterator i;
+  for (i = pending._added_tasks.begin (); i != pending._added_tasks.end (); ++i)
+    _changes.push_back (*i);
 
-  DEBUG_STR ("reading taskmods from file: ");
+  for (i = pending._modified_tasks.begin (); i != pending._modified_tasks.end (); ++i)
+    _changes.push_back (*i);
 
-  for ( ; start != input.end (); ++start)
-  {
-    line = *start;
+  for (i = completed._added_tasks.begin (); i != completed._added_tasks.end (); ++i)
+    _changes.push_back (*i);
 
-    if (line.substr (0, 4) == "time")
-    {
-      std::stringstream stream (line.substr (5));
-      long ts;
-      stream >> ts;
-
-      if (stream.fail ())
-        throw std::string (STRING_TDB2_UNDO_TIMESTAMP);
-
-      // 'time' is the first line of a modification
-      // thus we will (re)set the taskmod object
-      tmod_tmp.reset (ts);
-
-    }
-    else if (line.substr (0, 3) == "old")
-    {
-      tmod_tmp.setBefore (Task (line.substr (4)));
-
-    }
-    else if (line.substr (0, 3) == "new")
-    {
-      tmod_tmp.setAfter (Task (line.substr (4)));
-
-      // 'new' is the last line of a modification,
-      // thus we can push to the list
-      list.push_back (tmod_tmp);
-
-      assert (tmod_tmp.isValid ());
-      DEBUG_STR ("  taskmod complete");
-    }
-  }
-
-  DEBUG_STR ("DONE");
+  for (i = completed._modified_tasks.begin (); i != completed._modified_tasks.end (); ++i)
+    _changes.push_back (*i);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-void TDB2::merge (const std::string& mergeFile)
+void TDB2::get_changes (std::vector <Task>& changes)
 {
-  ///////////////////////////////////////
-  // Copyright 2010 - 2014, Johannes Schlatow.
-  ///////////////////////////////////////
-
-  // list of modifications that we want to add to the local database
-  std::list<Taskmod> mods;
-
-  // list of modifications that we want to add to the local history
-  std::list<Taskmod> mods_history;
-
-  // list of modifications on the local database
-  // has to be merged with mods to create the new undo.data
-  std::list<Taskmod> lmods;
-
-  // will contain the NEW undo.data
-  std::vector <std::string> undo_lines;
-
-  ///////////////////////////////////////
-  // initialize the files:
-
-  // load merge file (undo file of right/remote branch)
-  std::vector <std::string> r;
-  if (! File::read (mergeFile, r))
-    throw format (STRING_TDB2_UNREADABLE, mergeFile);
-
-  // file has to contain at least one entry
-  if (r.size () < 3)
-    throw std::string (STRING_TDB2_NO_CHANGES);
-
-  if (! undo._file.exists ())
-    undo._file.create ();
-
-  // load undo file (left/local branch)
-  std::vector <std::string> l;
-  if (! File::read (undo._file._data, l))
-    throw format (STRING_TDB2_UNREADABLE, undo._file._data);
-
-  std::string rline, lline;
-  std::vector <std::string>::iterator rit, lit;
-
-  // read first line
-  rit = r.begin ();
-  lit = l.begin ();
-
-  if (rit != r.end())
-    rline = *rit;
-  if (lit != l.end())
-    lline = *lit;
-
-  ///////////////////////////////////////
-  // find the branch-off point:
-
-  // first mods are not equal => assuming mergeFile starts at a
-  // later point in time
-  if (lline.compare (rline) == 0) {
-    std::vector<std::string>::const_iterator tmp_lit = lit;
-    std::vector<std::string>::const_iterator tmp_rit = rit;
-    tmp_lit++;
-    tmp_rit++;
-
-    int lookahead = 1;
-    if (tmp_lit->substr (0, 3) == "old") {
-      lookahead = 2;
-    }
-
-    while (lookahead--) {
-      if (tmp_lit->compare(*tmp_rit) != 0) {
-        break;
-      }
-      tmp_lit++;
-      tmp_rit++;
-    }
-
-    if (lookahead == -1) {
-      // at this point we know that the first lines are the same
-      undo_lines.push_back (lline + "\n");
-    }
-  }
-
-  // Add some color.
-  Color colorAdded    (context.config.get ("color.sync.added"));
-  Color colorChanged  (context.config.get ("color.sync.changed"));
-  Color colorRejected (context.config.get ("color.sync.rejected"));
-
-  // at this point we can assume: (lline==rline) || (lit == l.end())
-  // thus we search for the first non-equal lines or the EOF
-  bool found = false;
-  for (std::vector<std::string>::const_iterator tmp_lit = lit, tmp_rit = rit;
-       (tmp_lit != l.end ()) && (tmp_rit != r.end ());
-       ++tmp_lit, ++tmp_rit)
-  {
-    lline = *tmp_lit;
-    rline = *tmp_rit;
-
-    // found first non-matching lines?
-    if (lline.compare (rline) != 0)
-    {
-      found = true;
-      break;
-    }
-    else if (tmp_lit->substr (0, 3) == "---")
-    {
-      while (lit != tmp_lit)
-      {
-        ++lit;
-        ++rit;
-        undo_lines.push_back(*lit + "\n");
-      }
-      // at this point, all iterators (tmp_{lit,rit}, lit and rit) are at the same line
-    }
-  }
-
-  if (!found)
-  {
-    // set iterators to r.end() or l.end() if they point to the last line
-    if (++rit != r.end())
-      --rit;
-
-    if (++lit != l.end())
-      --lit;
-  }
-
-  ///////////////////////////////////////
-  // branch-off point found:
-  if (found)
-  {
-    DEBUG_STR_PART ("Branch-off point found at: ");
-    DEBUG_STR_END (lline);
-
-    std::list<Taskmod> rmods;
-
-    // helper lists
-    std::set<std::string> uuid_new, uuid_left;
-
-    // 1. read taskmods out of the remaining lines
-    readTaskmods (r, rit, rmods);
-    readTaskmods (l, lit, lmods);
-
-    // 2. move new uuids into mods
-    DEBUG_STR_PART ("adding new uuids (left) to skip list...");
-
-    // modifications on the left side are already in the database
-    // we just need them to merge conflicts, so we add the mods for
-    // new uuids to the skip-list 'uuid_left'
-    std::list<Taskmod>::iterator lmod_it;
-    for (lmod_it = lmods.begin (); lmod_it != lmods.end (); lmod_it++)
-    {
-      if (lmod_it->isNew ())
-      {
-/*
-        // TODO Don't forget L10N.
-        std::cout << "New local task                     "
-                  << (context.color () ? colorAdded.colorize (lmod_it->getUuid ()) : lmod_it->getUuid ())
-                  << "\n";
-*/
-
-        uuid_left.insert (lmod_it->getUuid ());
-      }
-    }
-
-    DEBUG_STR_END ("done");
-    DEBUG_STR_PART ("move new uuids (right) to redo list...");
-
-    // new items on the right side need to be inserted into the
-    // local database
-    std::list<Taskmod>::iterator rmod_it;
-    for (rmod_it = rmods.begin (); rmod_it != rmods.end (); )
-    {
-      // we have to save and increment the iterator because we may want to delete
-      // the object from the list
-      std::list<Taskmod>::iterator current = rmod_it++;
-      Taskmod tmod = *current;
-
-      if (uuid_left.find (tmod.getUuid ()) != uuid_left.end ())
-      {
-        // check whether the remote side has added a task with the same UUID
-        //  this happens if it inserted a modification with an older timestamp
-        //  into the undo.data and thereby moved the branch point to an earlier
-        //  point in time. Normally this case will be solved by the merge logic,
-        //  BUT if the UUID is considered new the merge logic will be skipped.
-        //
-        //  This flaw resulted in a couple of duplication issues and bloated
-        //  undo files (e.g. #1104).
-        //
-        //  This is just a "hack" which discards all the modifications of the
-        //  remote side to UUIDs that are considered new by both sides.
-        //  There may be more issues with the algorithm; probably a redesign
-        //  and proper encapsulation of the merge algorithm is due.
-
-        rmods.erase(current);
-      }
-      else if (tmod.isNew ())
-      {
-        // new uuid?
-/*
-        // TODO Don't forget L10N.
-        std::cout << "Adding new remote task             "
-                  << (context.color () ? colorAdded.colorize (tmod.getUuid ()) : tmod.getUuid ())
-                  << "\n";
-*/
-
-        uuid_new.insert (tmod.getUuid ());
-        mods.push_back (tmod);
-        rmods.erase (current);
-      }
-      else if (uuid_new.find (tmod.getUuid ()) != uuid_new.end ())
-      {
-        // uuid of modification was new
-        mods.push_back (tmod);
-        rmods.erase (current);
-      }
-    }
-
-    DEBUG_STR_END ("done");
-
-    ///////////////////////////////////////
-    // merge modifications:
-    DEBUG_STR ("Merging modifications:");
-
-    // we iterate backwards to resolve conflicts by timestamps (newest one wins)
-    std::list<Taskmod>::reverse_iterator lmod_rit;
-    std::list<Taskmod>::reverse_iterator rmod_rit;
-    for (lmod_rit = lmods.rbegin (); lmod_rit != lmods.rend (); ++lmod_rit)
-    {
-      Taskmod     tmod_l = *lmod_rit;
-      std::string uuid   = tmod_l.getUuid ();
-
-      DEBUG_STR ("  left uuid: " + uuid);
-
-      // skip if uuid had already been merged
-      if (uuid_left.find (uuid) == uuid_left.end ())
-      {
-        bool rwin = false;
-        bool lwin = false;
-        for (rmod_rit = rmods.rbegin (); rmod_rit != rmods.rend (); rmod_rit++)
-        {
-          Taskmod tmod_r = *rmod_rit;
-
-          DEBUG_STR ("    right uuid: " + tmod_r.getUuid ());
-          if (tmod_r.getUuid () == uuid)
-          {
-            DEBUG_STR ("    uuid match found for " + uuid);
-
-            // we already decided to take the mods from the right side
-            // but we have to find the first modification newer than
-            // the one on the left side to merge the history too
-            if (rwin)
-            {
-              DEBUG_STR ("    scanning right side");
-              if (tmod_r > tmod_l)
-                mods.push_front (tmod_r);
-
-              std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
-              rmods.erase (--tmp_it);
-              rmod_rit--;
-            }
-            else if (lwin)
-            {
-              DEBUG_STR ("    cleaning up right side");
-
-              // add tmod_r to local history
-              mods_history.push_front (tmod_r);
-
-              std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
-              rmods.erase (--tmp_it);
-              rmod_rit--;
-            }
-            else
-            {
-              // which one is newer?
-              if (tmod_r > tmod_l)
-              {
-                std::cout << format (STRING_TDB2_REMOTE_CHANGE,
-                                     (context.color () ? colorChanged.colorize (uuid) : uuid),
-                                     cutOff (tmod_r.getBefore ().get ("description"), 10))
-                          << "\n";
-
-                mods.push_front(tmod_r);
-
-                // delete tmod from right side
-                std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
-                rmods.erase (--tmp_it);
-                rmod_rit--;
-
-                rwin = true;
-              }
-              else
-              {
-                std::cout << format (STRING_TDB2_LOCAL_CHANGE,
-                                     (context.color () ? colorChanged.colorize (uuid) : uuid),
-                                     cutOff (tmod_l.getBefore ().get ("description"), 10))
-                          << "\n";
-
-                // inserting right mod into history of local database
-                // so that it can be restored later
-                // AND more important: create a history that looks the same
-                // as if we switched the roles 'remote' and 'local'
-
-                // thus we have to find the oldest change on the local branch that is not on remote branch
-                std::list<Taskmod>::iterator lmod_it;
-                std::list<Taskmod>::iterator last = lmod_it;
-                for (lmod_it = lmods.begin (); lmod_it != lmods.end (); ++lmod_it) {
-                  if ((*lmod_it).getUuid () == uuid) {
-                    last = lmod_it;
-                  }
-                }
-
-                if (tmod_l > tmod_r) { // local change is newer
-                  last->setBefore(tmod_r.getAfter ());
-
-                  // add tmod_r to local history
-                  lmods.push_back(tmod_r);
-                }
-                else { // both mods have equal timestamps
-                  // in this case the local branch wins as above, but the remote change with the
-                  // same timestamp will be discarded
-
-                  // find next (i.e. older) mod of this uuid on remote side
-                  std::list<Taskmod>::reverse_iterator rmod_rit2;
-                  for (rmod_rit2 = rmod_rit, ++rmod_rit2; rmod_rit2 != rmods.rend (); ++rmod_rit2) {
-                    Taskmod tmp_mod = *rmod_rit2;
-                    if (tmp_mod.getUuid () == uuid) {
-                      last->setBefore (tmp_mod.getAfter ());
-                      break;
-                    }
-                  }
-                }
-
-                // TODO feature: restore command? We would have to add a marker to the undo.file.
-
-                // delete tmod from right side
-                std::list<Taskmod>::iterator tmp_it = rmod_rit.base ();
-                rmods.erase (--tmp_it);
-                rmod_rit--;
-
-                // mark this uuid as merged
-                uuid_left.insert (uuid);
-                lwin = true;
-              }
-            }
-          }
-        } // for
-
-        if (rwin)
-        {
-          DEBUG_STR ("  concat the first match to left branch");
-          // concat the oldest (but still newer) modification on the right
-          // to the endpoint on the left
-          mods.front ().setBefore(tmod_l.getAfter ());
-        }
-      }
-    } // for
-
-    DEBUG_STR ("adding non-conflicting changes from the right branch");
-    mods.splice (mods.begin (), rmods);
-
-    DEBUG_STR ("sorting taskmod list");
-    mods.sort (compareTaskmod);
-    mods_history.sort (compareTaskmod);
-  }
-  else if (rit == r.end ())
-  {
-    // nothing happened on the remote branch
-    // local branch is up-to-date
-
-    // nothing happened on the local branch either
-
-    // break, to suppress autopush
-    if (lit == l.end ())
-    {
-      mods.clear ();
-      lmods.clear ();
-      throw std::string (STRING_TDB2_UP_TO_DATE);
-    }
-  }
-  else // lit == l.end ()
-  {
-    // nothing happened on the local branch
-/*
-    std::cout << "No local changes detected.\n";
-*/
-
-    // add remaining lines (remote branch) to the list of modifications
-/*
-    std::cout << "Remote changes detected.\n";
-*/
-    readTaskmods (r, rit, mods);
-  }
-
-  ///////////////////////////////////////
-  // Now apply the changes.
-  // redo command:
-
-  if (!mods.empty ())
-  {
-    std::vector <std::string> pending_lines;
-
-    std::vector <std::string> completed_lines;
-
-    if (! File::read (pending._file._data, pending_lines))
-      throw format (STRING_TDB2_UNREADABLE, pending._file._data);
-
-    if (! File::read (completed._file._data, completed_lines))
-      throw format (STRING_TDB2_UNREADABLE, completed._file._data);
-
-    // iterate over taskmod list
-    std::list<Taskmod>::iterator it;
-    for (it = mods.begin (); it != mods.end (); )
-    {
-      std::list<Taskmod>::iterator current = it++;
-      Taskmod tmod = *current;
-
-      // Modification to an existing task.
-      if (!tmod.isNew ())
-      {
-        std::string uuid = tmod.getUuid ();
-        Task::status statusBefore = tmod.getBefore().getStatus ();
-        Task::status statusAfter  = tmod.getAfter().getStatus ();
-
-        std::vector <std::string>::iterator it;
-
-        bool found = false;
-        if ( (statusBefore == Task::completed)
-          || (statusBefore == Task::deleted) )
-        {
-          // Find the same uuid in completed data
-          for (it = completed_lines.begin (); it != completed_lines.end (); ++it)
-          {
-            if (it->find ("uuid:\"" + uuid) != std::string::npos)
-            {
-              // Update the completed record.
-/*
-              std::cout << "Modifying                     "
-                        << (context.color () ? colorChanged.colorize (uuid) : uuid)
-                        << "\n";
-*/
-
-              std::string newline = tmod.getAfter ().composeF4 ();
-
-              // does the tasks move to pending data?
-              // this taskmod will not arise from
-              // normal usage of task, but those kinds of
-              // taskmods may be constructed to merge databases
-              if ( (statusAfter != Task::completed)
-                && (statusAfter != Task::deleted) )
-              {
-                // insert task into pending data
-                pending_lines.push_back (newline);
-
-                // remove task from completed data
-                completed_lines.erase (it);
-
-              }
-              else
-              {
-                // replace the current line
-                *it = newline;
-              }
-
-              found = true;
-              break;
-            }
-          }
-        }
-        else
-        {
-          // Find the same uuid in the pending data.
-          for (it = pending_lines.begin (); it != pending_lines.end (); ++it)
-          {
-            if (it->find ("uuid:\"" + uuid) != std::string::npos)
-            {
-              // Update the pending record.
-              std::cout << format (STRING_TDB2_REMOTE_CHANGE,
-                                   (context.color () ? colorChanged.colorize (uuid) : uuid),
-                                   cutOff (tmod.getBefore ().get ("description"), 10))
-                        << "\n";
-
-              std::string newline = tmod.getAfter ().composeF4 ();
-
-              // does the tasks move to completed data
-              if ( (statusAfter == Task::completed)
-                || (statusAfter == Task::deleted) )
-              {
-                // insert task into completed data
-                completed_lines.push_back (newline);
-
-                // remove task from pending data
-                pending_lines.erase (it);
-
-              }
-              else
-              {
-                // replace the current line
-                *it = newline;
-              }
-
-              found = true;
-              break;
-            }
-          }
-        }
-
-        if (!found)
-        {
-          std::cout << format (STRING_TDB2_MISSING,
-                               (context.color () ? colorRejected.colorize (uuid) : uuid),
-                               cutOff (tmod.getBefore ().get ("description"), 10))
-                    << "\n";
-          mods.erase (current);
-        }
-      }
-      else
-      {
-        // Check for dups.
-        std::string uuid = tmod.getAfter ().get ("uuid");
-
-        // Find the same uuid in the pending data.
-        bool found = false;
-        std::vector <std::string>::iterator pit;
-        for (pit = pending_lines.begin (); pit != pending_lines.end (); ++pit)
-        {
-          if (pit->find ("uuid:\"" + uuid) != std::string::npos)
-          {
-            found = true;
-            break;
-          }
-        }
-
-        if (!found)
-        {
-          std::cout << format (STRING_TDB2_MERGING,
-                               (context.color () ? colorAdded.colorize (uuid) : uuid),
-                               cutOff (tmod.getAfter ().get ("description"), 10))
-                    << "\n";
-
-          pending_lines.push_back (tmod.getAfter ().composeF4 ());
-        }
-        else
-        {
-          mods.erase (current);
-        }
-      }
-    }
-
-    // write pending file
-    if (! File::write (pending._file._data, pending_lines))
-      throw format (STRING_TDB2_UNWRITABLE, pending._file._data);
-
-    // write completed file
-    if (! File::write (completed._file._data, completed_lines))
-      throw format (STRING_TDB2_UNWRITABLE, completed._file._data);
-  }
-
-  if (!mods.empty() || !lmods.empty() || !mods_history.empty()) {
-    // at this point undo contains the lines up to the branch-off point
-    // now we merge mods (new modifications from mergefile)
-    // with lmods (part of old undo.data)
-    lmods.sort(compareTaskmod);
-    mods.merge (lmods, compareTaskmod);
-    mods.merge (mods_history, compareTaskmod);
-
-    // generate undo.data format
-    std::list<Taskmod>::iterator it;
-    for (it = mods.begin (); it != mods.end (); it++)
-      undo_lines.push_back(it->toString ());
-
-    // write undo file
-    if (! File::write (undo._file._data, undo_lines, false))
-      throw format (STRING_TDB2_UNWRITABLE, undo._file._data);
-  }
-
-  // delete objects
-  lmods.clear ();
-  mods.clear ();
-  mods_history.clear ();
+  changes = _changes;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1538,6 +985,9 @@ void TDB2::show_diff (
     view.add (Column::factory ("string", STRING_TDB2_UNDO_PRIOR));
     view.add (Column::factory ("string", STRING_TDB2_UNDO_CURRENT));
 
+    Color label (context.config.get ("color.label"));
+    view.colorHeader (label);
+
     Task after (current);
 
     if (prior != "")
@@ -1728,7 +1178,6 @@ void TDB2::show_diff (
               << view.render ()
               << "\n";
   }
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1749,18 +1198,21 @@ int TDB2::gc ()
   // Allowed as an override, but not recommended.
   if (context.config.getBoolean ("gc"))
   {
-    std::vector <Task> pending_tasks   = pending.get_tasks ();
+    std::vector <Task> pending_tasks = pending.get_tasks ();
+
+    // TODO Thread.
     std::vector <Task> completed_tasks = completed.get_tasks ();
 
+    // TODO Assume pending < completed, therefore there is room here to process
+    //      data before joining with the completed.data thread.
+
     bool pending_changes = false;
     bool completed_changes = false;
-
     std::vector <Task> pending_tasks_after;
     std::vector <Task> completed_tasks_after;
 
     // Reduce unnecessary allocation/copies.
     pending_tasks_after.reserve (pending_tasks.size ());
-    completed_tasks_after.reserve (completed_tasks.size ());
 
     // Scan all pending tasks, looking for any that need to be relocated to
     // completed, or need to be 'woken'.
@@ -1797,6 +1249,11 @@ int TDB2::gc ()
       }
     }
 
+    // TODO Join completed.data thread.
+
+    // Reduce unnecessary allocation/copies.
+    completed_tasks_after.reserve (completed_tasks.size ());
+
     // Scan all completed tasks, looking for any that need to be relocated to
     // pending.
     for (task = completed_tasks.begin ();
@@ -1913,6 +1370,14 @@ bool TDB2::get (const std::string& uuid, Task& task)
          completed.get (uuid, task);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Locate task by UUID, wherever it is.
+bool TDB2::has (const std::string& uuid)
+{
+  return pending.has (uuid) ||
+         completed.has (uuid);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 const std::vector <Task> TDB2::siblings (Task& task)
 {
@@ -2038,7 +1503,7 @@ void TDB2::dump ()
     context.debug (completed.dump ());
     context.debug (undo.dump ());
     context.debug (backlog.dump ());
-    context.debug ("");
+    context.debug (" ");
   }
 }
 
