@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2014, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -33,8 +32,8 @@
 #include <vector>
 #include <string>
 #include <strings.h>
-#include <ctype.h>
 #include <Context.h>
+#include <Lexer.h>
 #include <math.h>
 #include <util.h>
 #include <text.h>
@@ -61,47 +60,6 @@ void wrapText (
     lines.push_back (line);
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// UTF-8
-void splitq (
-  std::vector<std::string>& results,
-  const std::string& input,
-  const char delimiter)
-{
-  results.clear ();
-
-  std::string::size_type start = 0;
-  std::string::size_type i = 0;
-  std::string word;
-  bool in_quote = false;
-  char quote;
-
-  while (utf8_next_char (input, i))
-  {
-    if (in_quote)
-    {
-      if (input[i] == quote)
-        in_quote = false;
-    }
-    else
-    {
-      if (input[i] == delimiter)
-      {
-        results.push_back (unquoteText (input.substr (start, i - start)));
-        start = i + 1;
-      }
-      else if (input[i] == '\'' ||
-               input[i] == '"')
-      {
-        quote = input[i];
-        in_quote = true;
-      }
-    }
-  }
-
-  results.push_back (unquoteText (input.substr (start)));
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 void split (
   std::vector<std::string>& results,
@@ -121,26 +79,6 @@ void split (
     results.push_back (input.substr (start));
 }
 
-////////////////////////////////////////////////////////////////////////////////
-void split_minimal (
-  std::vector<std::string>& results,
-  const std::string& input,
-  const char delimiter)
-{
-  results.clear ();
-  std::string::size_type start = 0;
-  std::string::size_type i;
-  while ((i = input.find (delimiter, start)) != std::string::npos)
-  {
-    if (i != start)
-      results.push_back (input.substr (start, i - start));
-    start = i + 1;
-  }
-
-  if (start < input.length ())
-    results.push_back (input.substr (start));
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 void split (
   std::vector<std::string>& results,
@@ -290,6 +228,7 @@ int longestLine (const std::string& input)
   return longest;
 }
 
+////////////////////////////////////////////////////////////////////////////////
 // Walk the input text looking for a break point.  A break point is one of:
 //   - EOS
 //   - \n
@@ -297,7 +236,7 @@ int longestLine (const std::string& input)
 //   - last punctuation (, ; . :) before 'length' characters, even if not
 //     followed by a space
 //   - first 'length' characters
-
+//
 // text       "one two three\n  four"
 // bytes       0123456789012 3456789
 // characters  1234567890a23 4567890
@@ -411,7 +350,7 @@ std::string commify (const std::string& data)
   int i;
   for (int i = 0; i < (int) data.length (); ++i)
   {
-    if (isdigit (data[i]))
+    if (Lexer::isDigit (data[i]))
       end = i;
 
     if (data[i] == '.')
@@ -429,11 +368,11 @@ std::string commify (const std::string& data)
     int consecutiveDigits = 0;
     for (; i >= 0; --i)
     {
-      if (isdigit (data[i]))
+      if (Lexer::isDigit (data[i]))
       {
         result += data[i];
 
-        if (++consecutiveDigits == 3 && i && isdigit (data[i - 1]))
+        if (++consecutiveDigits == 3 && i && Lexer::isDigit (data[i - 1]))
         {
           result += ',';
           consecutiveDigits = 0;
@@ -453,11 +392,11 @@ std::string commify (const std::string& data)
     int consecutiveDigits = 0;
     for (; i >= 0; --i)
     {
-      if (isdigit (data[i]))
+      if (Lexer::isDigit (data[i]))
       {
         result += data[i];
 
-        if (++consecutiveDigits == 3 && i && isdigit (data[i - 1]))
+        if (++consecutiveDigits == 3 && i && Lexer::isDigit (data[i - 1]))
         {
           result += ',';
           consecutiveDigits = 0;
@@ -484,14 +423,6 @@ std::string lowerCase (const std::string& input)
   return output;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-std::string upperCase (const std::string& input)
-{
-  std::string output = input;
-  std::transform (output.begin (), output.end (), output.begin (), toupper);
-  return output;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 std::string ucFirst (const std::string& input)
 {
@@ -535,154 +466,18 @@ const char* optionalBlankLine ()
   return context.verbose ("blank") ? newline : noline;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-void guess (
-  const std::string& type,
-  std::vector<std::string>& options,
-  std::string& candidate)
-{
-  std::vector <std::string> matches;
-  autoComplete (candidate, options, matches,
-                context.config.getInteger ("abbreviation.minimum"));
-  if (1 == matches.size ())
-    candidate = matches[0];
-
-  else if (0 == matches.size ())
-    candidate = "";
-
-  else
-  {
-    std::sort (matches.begin (), matches.end ());
-
-    std::string error = format (STRING_TEXT_AMBIGUOUS, type, candidate);
-    for (size_t i = 0; i < matches.size (); ++i)
-    {
-      if (i)
-        error += ", ";
-      error += matches[i];
-    }
-
-    throw error;
-  }
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 bool nontrivial (const std::string& input)
 {
-  for (size_t i = 0; i < input.length (); ++i)
-    if (!isspace (input[i]))
+  std::string::size_type i = 0;
+  int character;
+  while ((character = utf8_next_char (input, i)))
+    if (! Lexer::isWhitespace (character))
       return true;
 
   return false;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-bool digitsOnly (const std::string& input)
-{
-  for (size_t i = 0; i < input.length (); ++i)
-    if (!isdigit (input[i]))
-      return false;
-
-  return true;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool noSpaces (const std::string& input)
-{
-  for (size_t i = 0; i < input.length (); ++i)
-    if (isspace (input[i]))
-      return false;
-
-  return true;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool noVerticalSpace (const std::string& input)
-{
-  if (input.find_first_of ("\n\r\f") != std::string::npos)
-    return false;
-
-  return true;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-//          Input: hello, world
-// Result for pos: y......y....
-bool isWordStart (const std::string& input, std::string::size_type pos)
-{
-  // Short circuit: no input means no word start.
-  if (input.length () == 0)
-    return false;
-
-  // If pos is the first non space/punct character of the string.
-  if (pos == 0 && !isspace (input[pos]) && !isPunctuation (input[pos]))
-    return true;
-
-  // If pos is not the first alphanumeric character, but there is a preceding
-  // space/punct character.
-  if (pos > 0 && !isspace (input[pos]) && !isPunctuation (input[pos])
-              && (isspace (input[pos - 1]) || isPunctuation (input[pos - 1])))
-    return true;
-
-  return false;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-//          Input: hello, world
-// Result for pos: ....y......y
-bool isWordEnd (const std::string& input, std::string::size_type pos)
-{
-  // Short circuit: no input means no word end.
-  if (input.length () == 0)
-    return false;
-
-  // If pos is the last alphanumeric character of the string.
-  if (pos == input.length () - 1 && !isspace (input[pos]) && !isPunctuation (input[pos]))
-    return true;
-
-  // If pos is not the last alphanumeric character, but there is a following
-  // non-alphanumeric character.
-  if (pos < input.length () - 1 && !isspace (input[pos]) && !isPunctuation (input[pos])
-                                && (isspace (input[pos + 1]) || isPunctuation (input[pos + 1])))
-    return true;
-
-  return false;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-//          Input: hello, world
-// Result for pos: ....y......y
-//
-//          Input: (one) two
-// Result for pos: y..yy...y
-bool isTokenEnd (const std::string& input, std::string::size_type pos)
-{
-  // Delegate.
-  if (isWordEnd (input, pos))
-    return true;
-
-  // Punctuation divides tokens.
-  if (pos < input.length () && isPunctuation (input[pos]))
-    return true;
-
-  return false;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// Override of ispunct, that considers #, $ and @ not to be punctuation.
-//
-// ispunct:      ! " # $ % & ' ( ) * + , - . / : ; < = > ? @ [ \ ] ^ _ ` { | } ~
-// Punctuation:  ! "     % & ' ( ) * + , - . / : ; < = > ?   [ \ ] ^ _ ` { | } ~
-// delta:            # $                                   @
-//
-bool isPunctuation (char c)
-{
-  if (c == '@' || c == '#' || c == '$')
-    return false;
-
-  return ispunct (c);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 bool compare (
   const std::string& left,
@@ -829,22 +624,6 @@ int strippedLength (const std::string& input)
   return count;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// Truncates a long line, and include a two-character ellipsis.
-std::string cutOff (const std::string& str, std::string::size_type len)
-{
-  if (str.length () > len)
-  {
-    return (str.substr (0, len - 2) + "..");
-  }
-  else
-  {
-    std::string res = str;
-    res.resize (len, ' ');
-    return res;
-  }
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 const std::string format (char value)
 {
@@ -986,6 +765,20 @@ const std::string format (
   return output;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  int arg2,
+  const std::string& arg3)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", format (arg2));
+  replace_positional (output, "{3}", arg3);
+  return output;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 const std::string format (
   const std::string& fmt,
@@ -998,6 +791,36 @@ const std::string format (
   return output;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  const std::string& arg2,
+  const std::string& arg3)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", arg2);
+  replace_positional (output, "{3}", arg3);
+  return output;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  const std::string& arg2,
+  const std::string& arg3,
+  const std::string& arg4)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", arg2);
+  replace_positional (output, "{3}", arg3);
+  replace_positional (output, "{4}", arg4);
+  return output;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 const std::string format (
   const std::string& fmt,
@@ -1024,6 +847,22 @@ const std::string format (
   return output;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  int arg1,
+  int arg2,
+  int arg3,
+  int arg4)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", format (arg1));
+  replace_positional (output, "{2}", format (arg2));
+  replace_positional (output, "{3}", format (arg3));
+  replace_positional (output, "{4}", format (arg4));
+  return output;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 const std::string format (
   const std::string& fmt,
@@ -1050,6 +889,22 @@ const std::string format (
   return output;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+const std::string format (
+  const std::string& fmt,
+  const std::string& arg1,
+  const std::string& arg2,
+  const std::string& arg3,
+  const std::string& arg4)
+{
+  std::string output = fmt;
+  replace_positional (output, "{1}", arg1);
+  replace_positional (output, "{2}", arg2);
+  replace_positional (output, "{3}", arg3);
+  replace_positional (output, "{4}", arg4);
+  return output;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 std::string leftJustify (const int input, const int width)
 {
@@ -1084,7 +939,11 @@ std::string rightJustify (const int input, const int width)
 ////////////////////////////////////////////////////////////////////////////////
 std::string rightJustify (const std::string& input, const int width)
 {
-  return std::string (width - utf8_text_width (input), ' ') + input;
+  unsigned int len = utf8_text_width (input);
+  return (((unsigned int) width > len)
+           ? std::string (width - len, ' ')
+           : "")
+         + input;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
