@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2014, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2015, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +37,7 @@ Context context;
 ////////////////////////////////////////////////////////////////////////////////
 int main (int argc, char** argv)
 {
-  UnitTest t (264);
+  UnitTest t (205);
 
   // Ensure environment has no influence.
   unsetenv ("TASKDATA");
@@ -125,9 +124,6 @@ int main (int argc, char** argv)
   t.is (items[0], "",              "split '-' '-' -> [0] ''");
   t.is (items[1], "",              "split '-' '-' -> [1] ''");
 
-  split_minimal (items, unsplit, '-');
-  t.is (items.size (), (size_t) 0, "split '-' '-' ->");
-
   unsplit = "-a-bc-def";
   split (items, unsplit, '-');
   t.is (items.size (), (size_t) 4, "split '-a-bc-def' '-' -> '' 'a' 'bc' 'def'");
@@ -136,12 +132,6 @@ int main (int argc, char** argv)
   t.is (items[2], "bc",            "split '-a-bc-def' '-' -> [2] 'bc'");
   t.is (items[3], "def",           "split '-a-bc-def' '-' -> [3] 'def'");
 
-  split_minimal (items, unsplit, '-');
-  t.is (items.size (), (size_t) 3, "split '-a-bc-def' '-' -> 'a' 'bc' 'def'");
-  t.is (items[0], "a",             "split '-a-bc-def' '-' -> [1] 'a'");
-  t.is (items[1], "bc",            "split '-a-bc-def' '-' -> [2] 'bc'");
-  t.is (items[2], "def",           "split '-a-bc-def' '-' -> [3] 'def'");
-
   // void split (std::vector<std::string>& results, const std::string& input, const std::string& delimiter)
   unsplit = "";
   split (items, unsplit, "--");
@@ -173,17 +163,6 @@ int main (int argc, char** argv)
   t.is (items[1], "two",           "split 'one\\ntwo\\nthree' -> [1] 'two'");
   t.is (items[2], "three",         "split 'one\\ntwo\\nthree' -> [2] 'three'");
 
-  // void splitq (std::vector<std::string>&, const std::string&, const char);
-  unsplit = "one 'two' '' 'three four' \"five six seven\" eight'nine ten'";
-  splitq (items, unsplit, ' ');
-  t.is (items.size () , (size_t) 6,  "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten'");
-  t.is (items[0], "one",             "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten' -> [0] 'one'");
-  t.is (items[1], "two",             "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten' -> [1] 'two'");
-  t.is (items[2], "",                "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten' -> [2] ''");
-  t.is (items[3], "three four",      "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten' -> [3] 'three four'");
-  t.is (items[4], "five six seven",  "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten' -> [4] 'five six seven'");
-  t.is (items[5], "eight'nine ten'", "splitq 'one \\'two\\' \\'\\' \\'three four\\' \"five six seven\" eight'nine ten' -> [4] 'eight\\'nine ten\\''");
-
   // void join (std::string& result, const std::string& separator, const std::vector<std::string>& items)
   std::vector <std::string> unjoined;
   std::string joined;
@@ -300,10 +279,6 @@ int main (int argc, char** argv)
   t.is (lowerCase (""),            "",            "lowerCase '' -> ''");
   t.is (lowerCase ("pre01_:POST"), "pre01_:post", "lowerCase 'pre01_:POST' -> 'pre01_:post'");
 
-  // std::string upperCase (const std::string& input)
-  t.is (upperCase (""),            "",            "upperCase '' -> ''");
-  t.is (upperCase ("pre01_:POST"), "PRE01_:POST", "upperCase 'pre01_:POST' -> 'PRE01_:POST'");
-
   // bool nontrivial (const std::string&);
   t.notok (nontrivial (""),                       "nontrivial '' -> false");
   t.notok (nontrivial ("   "),                    "nontrivial '   ' -> false");
@@ -315,65 +290,6 @@ int main (int argc, char** argv)
   t.ok    (nontrivial ("  \t\ta"),                "nontrivial '  \\t\\ta' -> true");
   t.ok    (nontrivial ("a\t\t  "),                "nontrivial 'a\\t\\t  ' -> true");
 
-  // bool digitsOnly (const std::string&);
-  t.ok    (digitsOnly (""),                       "digitsOnly '' -> true");
-  t.ok    (digitsOnly ("0"),                      "digitsOnly '0' -> true");
-  t.ok    (digitsOnly ("123"),                    "digitsOnly '123' -> true");
-  t.notok (digitsOnly ("12fa"),                   "digitsOnly '12fa' -> false");
-
-  // bool noSpaces (const std::string&);
-  t.ok    (noSpaces (""),                         "noSpaces '' -> true");
-  t.ok    (noSpaces ("a"),                        "noSpaces 'a' -> true");
-  t.ok    (noSpaces ("abc"),                      "noSpaces 'abc' -> true");
-  t.notok (noSpaces (" "),                        "noSpaces ' ' -> false");
-  t.notok (noSpaces ("ab cd"),                    "noSpaces 'ab cd' -> false");
-
-  // bool noVerticalSpace (const std::string&);
-  t.ok    (noVerticalSpace (""),                  "noVerticalSpace '' -> true");
-  t.ok    (noVerticalSpace ("a"),                 "noVerticalSpace 'a' -> true");
-  t.ok    (noVerticalSpace ("abc"),               "noVerticalSpace 'abc' -> true");
-  t.notok (noVerticalSpace ("a\nb"),              "noVerticalSpace 'a\\nb' -> false");
-  t.notok (noVerticalSpace ("a\rb"),              "noVerticalSpace 'a\\rb' -> false");
-  t.notok (noVerticalSpace ("a\fb"),              "noVerticalSpace 'a\\fb' -> false");
-
-  text = "Hello, world.";
-  //      0123456789012
-  //      s   e  s   e
-
-  // bool isWordStart (const std::string&, std::string::size_type);
-  t.notok (isWordStart ("", 0),                   "isWordStart (\"\", 0) -> false");
-  t.ok    (isWordStart ("foo", 0),                "isWordStart (\"foo\", 0) -> true");
-  t.ok    (isWordStart (text, 0),                 "isWordStart (\"Hello, world.\", 0) -> true");
-  t.notok (isWordStart (text, 1),                 "isWordStart (\"Hello, world.\", 1) -> false");
-  t.notok (isWordStart (text, 2),                 "isWordStart (\"Hello, world.\", 2) -> false");
-  t.notok (isWordStart (text, 3),                 "isWordStart (\"Hello, world.\", 3) -> false");
-  t.notok (isWordStart (text, 4),                 "isWordStart (\"Hello, world.\", 4) -> false");
-  t.notok (isWordStart (text, 5),                 "isWordStart (\"Hello, world.\", 5) -> false");
-  t.notok (isWordStart (text, 6),                 "isWordStart (\"Hello, world.\", 6) -> false");
-  t.ok    (isWordStart (text, 7),                 "isWordStart (\"Hello, world.\", 7) -> true");
-  t.notok (isWordStart (text, 8),                 "isWordStart (\"Hello, world.\", 8) -> false");
-  t.notok (isWordStart (text, 9),                 "isWordStart (\"Hello, world.\", 9) -> false");
-  t.notok (isWordStart (text, 10),                "isWordStart (\"Hello, world.\", 10) -> false");
-  t.notok (isWordStart (text, 11),                "isWordStart (\"Hello, world.\", 11) -> false");
-  t.notok (isWordStart (text, 12),                "isWordStart (\"Hello, world.\", 12) -> false");
-
-  // bool isWordEnd (const std::string&, std::string::size_type);
-  t.notok (isWordEnd ("", 0),                     "isWordEnd (\"\", 0) -> false");
-  t.ok    (isWordEnd ("foo", 2),                  "isWordEnd (\"foo\", 2) -> true");
-  t.notok (isWordEnd (text, 0),                   "isWordEnd (\"Hello, world.\", 0) -> false");
-  t.notok (isWordEnd (text, 1),                   "isWordEnd (\"Hello, world.\", 1) -> false");
-  t.notok (isWordEnd (text, 2),                   "isWordEnd (\"Hello, world.\", 2) -> false");
-  t.notok (isWordEnd (text, 3),                   "isWordEnd (\"Hello, world.\", 3) -> false");
-  t.ok    (isWordEnd (text, 4),                   "isWordEnd (\"Hello, world.\", 4) -> true");
-  t.notok (isWordEnd (text, 5),                   "isWordEnd (\"Hello, world.\", 5) -> false");
-  t.notok (isWordEnd (text, 6),                   "isWordEnd (\"Hello, world.\", 6) -> false");
-  t.notok (isWordEnd (text, 7),                   "isWordEnd (\"Hello, world.\", 7) -> false");
-  t.notok (isWordEnd (text, 8),                   "isWordEnd (\"Hello, world.\", 8) -> false");
-  t.notok (isWordEnd (text, 9),                   "isWordEnd (\"Hello, world.\", 9) -> false");
-  t.notok (isWordEnd (text, 10),                  "isWordEnd (\"Hello, world.\", 10) -> false");
-  t.ok    (isWordEnd (text, 11),                  "isWordEnd (\"Hello, world.\", 11) -> true");
-  t.notok (isWordEnd (text, 12),                  "isWordEnd (\"Hello, world.\", 12) -> false");
-
   // bool compare (const std::string&, const std::string&, bool caseless = false);
   // Make sure degenerate cases are handled.
   t.ok    (compare ("", ""),    "'' == ''");
